program Pokrivanje; { RTK 1989.3.3 }
const MaxTock = 10; 
type 
  { Oh what a tangled web we weave,
    When first we practice to deceive!
             -- Scott, Marmion, VI 523-3 }
  PPremica = ^TPremica; 
  PTockaNaPremici = ^TTockaNaPremici; 
  TTocka = record 
    x, y: real;              { koordinati } 
    pp: PTockaNaPremici;     { prva premica, ki vsebuje to toko } 
  end;
  TTockaNaPremici = record 
    it: integer;             { indeks toke } 
    p: PPremica;             { kazalec na premico } 
    pt, nt: PTockaNaPremici; { prejnja in naslednja toka na tej premici } 
    np: PTockaNaPremici;     { naslednja premica, ki vsebuje to toko } 
  end; 
  TPremica = record 
    pt: PTockaNaPremici;     { prva toka na tej premici } 
    n: integer;              { tevilo tok na tej premici } 
    pp, np: PPremica;        { prejnja in naslednja premica z n tokami } 
  end; 

var 
  { Premice[k] je seznam premic, ki pokrivajo k tok. }
  Premice: array[1..MaxTock] of PPremica; 
  t: array[1..MaxTock] of TTocka; { toke } 
  n: integer;   { tevilo tok } 

{ Ali lei k na premici, ki jo doloata i in j? } 
function NaPremici(i, j, k: integer): boolean; 
const eps = 1e-4; 
var dx, dy, ux, uy, v, d2: real; 
begin
  dx := t[j].x - t[i].x; dy := t[j].y - t[i].y; 
  ux := t[k].x - t[i].x; uy := t[k].y - t[i].y; 
  v := dx * uy - dy * ux; d2 := dx * dx + dy * dy; 
  NaPremici := (d2 > 0) and (v * v <= eps * eps * d2); 
end; {NaPremici} 

{ Doda v sezname podatek, da premica p pokriva toko it. } 
procedure Dodaj(it: integer; p: PPremica); 
var tp: PTockaNaPremici; 
begin
  New(tp); tp^.it := it; tp^.p := p; p^.n := p^.n + 1; 
  { Dodajmo jo v seznam tok, ki jih pokriva premica p. } 
  tp^.pt := nil; tp^.nt := p^.pt; p^.pt := tp; 
  if tp^.nt <> nil then tp^.nt^.pt := tp;
  { Dodajmo jo v seznam premic, ki pokrivajo toko it. } 
  tp^.np := t[it].pp; t[it].pp := tp; 
end; {Dodaj} 

{ Zbrie toko it iz vseh premic, ki jo pokrivajo. } 
procedure Brisi(it: integer);
var tp: PTockaNaPremici; p: PPremica; n: integer; 
begin
  while t[it].pp <> nil do begin 
    tp := t[it].pp; t[it].pp := tp^.np; p := tp^.p; 
    { Zbriimo toko it iz seznama tok, ki jih pokriva premica p. } 
    if tp^.pt = nil then p^.pt := tp^.nt else tp^.pt^.nt := tp^.nt; 
    if tp^.nt <> nil then tp^.nt^.pt := tp^.pt; 
    Dispose(tp); n := p^.n; p^.n := p^.n - 1; 
    { Zbriimo p iz seznama premic, ki pokrivajo n tok. } 
    if p^.pp = nil then Premice[n] := p^.np else p^.pp^.np := p^.np; 
    if p^.np <> nil then p^.np^.pp := p^.pp; 
    if n > 1 then begin 
      { Dodajmo p v seznam premic, ki pokrivajo n - 1 tok. } 
      p^.pp := nil; p^.np := Premice[n - 1]; 
      if p^.np <> nil then p^.np^.pp := p; 
      Premice[n - 1] := p;  
    end; {if}
  end; {while}
end; {Brisi} 

var i, j, k, pn, StPremic: integer; p: PPremica; F: text;
begin

  { Preberimo koordinate tok.  e smo iz ukazne vrstice
    dobili ime datoteke, jih preberimo od tam, sicer pa jih
    bo moral vnesti uporabnik zdajle. } 
  if ParamCount > 0 then begin
    Assign(F, ParamStr(1)); Reset(F); ReadLn(F, n);
    for i := 1 to n do ReadLn(F, t[i].x, t[i].y);
    Close(F);
  end else begin
    Write('tevilo tok: '); ReadLn(n); 
    for i := 1 to n do begin 
      Write('X[', i, ']: '); Read(t[i].x); 
      Write(' Y[', i, ']: '); ReadLn(t[i].y); 
      t[i].pp := nil; 
    end; {for}
  end; {if}
  
  { Poiimo vse premice. } 
  for i := 1 to n do Premice[i] := nil; 
  for i := 1 to n - 1 do for j := i + 1 to n do begin 
    New(p); p^.n := 0; p^.pt := nil; p^.np := nil; p^.pp := nil; 
    { Poglejmo, katere toke pokriva premica skozi t[i] in t[j]. }
    for k := 1 to n do if NaPremici(i, j, k) then Dodaj(k, p); 
    { Dodajmo premico v seznam premic, ki pokrivajo p^.n tok. } 
    if Premice[p^.n] <> nil then Premice[p^.n]^.pp := p; 
    p^.np := Premice[p^.n]; Premice[p^.n] := p; 
  end; {for}
  if n = 1 then begin { Edino toko pokrijemo z eno samo premico. }
    New(p); p^.n := 0; p^.pt := nil; p^.np := nil; p^.pp := nil;
    Dodaj(1, p); Premice[p^.n] := p; 
  end; {if}

  { Poreno izbirajmo premice. } 
  pn := n; StPremic := 0; 
  while pn > 0 do begin 
    while Premice[pn] <> nil do begin 
      { p je ena od premic, ki pokrivajo najve tok. } 
      p := Premice[pn]; Write('Premica skozi'); 
      while p^.pt <> nil do begin { Pobriimo toke, ki jih p pokriva. } 
        Write(' T', p^.pt^.it);
        with t[p^.pt^.it] do Write('(', x:0:0, ', ', y:0:0, ')');
        Brisi(p^.pt^.it);
      end; {while}
      Dispose(p); WriteLn; StPremic := StPremic + 1; 
    end; {while}
    { Premic, ki bi pokrivale pn tok, ni ve; posvetimo se tistim, 
      ki pokrivajo pn - 1 ali manj tok. }
    pn := pn - 1;       
  end; {while}
  
  WriteLn('tevilo uporabljenih premic: ', StPremic); 
  
end. {Pokrivanje} 