program Poplavljanje; { RTK 1990.1.3 }
uses Crt;

{ Naloga pravi, da ima vsak piksel svoj procesor.  Ker pa
  bo moral tale na program tei na navadnem enoprocesorskem
  raunalniku, bomo dogajanje simulirali.  Podprogram
  SimulirajProcesor dobi zato kot parameter tevilko procesorja
  in jo mora tudi podati naprej podprogramom, kot so
  PobarvajMe in podobni.  SimulirajProcesor naj bi kaj malega
  postoril, nato pa se hitro vrnil, da bo lahko glavni program
  podobno poklical SimulirajProcesor e za vse ostale procesorje. }

  const MejnaBarva = Ord('M');
  const XMax = 3; YMax = 3; { velikost vzorca }
  
  function Vzorec(x, y: integer): integer; forward;
  function JeSporocilo(StProcesorja: integer): integer; forward;
  function Sprejmi(StProcesorja, Zveza: integer): integer; forward;
  procedure Poslji(StProcesorja, Zveza, Sporocilo: integer); forward;
  function MojaBarva(StProcesorja: integer): integer; forward;
  procedure PobarvajMe(StProcesorja, Barva: integer); forward;
  
procedure SimulirajProcesor(StProcesorja: integer; var Stanje: integer); 
var 
  Zveza: integer;       { od kod smo dobili sporoilo } 
  MojX, MojY: integer;  { kje v rastru smo } 
  NovaBarva: integer;   { ustrezna barva iz vzorca } 
begin
  if Stanje = 0 then begin
    if MojaBarva(StProcesorja) <> MejnaBarva then begin 
      { akamo na sporoilo. } 
      Zveza := JeSporocilo(StProcesorja); 
      if Zveza = 0 then exit;
      { Sprejmemo svoje koordinate. }
      MojX := Sprejmi(StProcesorja, Zveza); 
      MojY := Sprejmi(StProcesorja, Zveza);
      NovaBarva := Vzorec(MojX mod XMax, MojY mod YMax); 
      if NovaBarva <> MojaBarva(StProcesorja) then begin 
        PobarvajMe(StProcesorja, NovaBarva); { pobarvamo se } 
        { Obvestimo sosede, razen tistega, ki je obvestil nas. } 
        if Zveza <> 1 then begin Poslji(StProcesorja, 1, MojX); Poslji(StProcesorja, 1, MojY - 1) end;
        if Zveza <> 2 then begin Poslji(StProcesorja, 2, MojX + 1); Poslji(StProcesorja, 2, MojY) end;
        if Zveza <> 3 then begin Poslji(StProcesorja, 3, MojX); Poslji(StProcesorja, 3, MojY + 1) end;
        if Zveza <> 4 then begin Poslji(StProcesorja, 4, MojX - 1); Poslji(StProcesorja, 4, MojY) end;
      end; {if}
    end; {if}
    Stanje := 1;
  end; {if}
  { Mogoe nam bo e kdo ukazal, naj se pobarvamo. } 
  repeat
    Zveza := JeSporocilo(StProcesorja); 
    if Zveza = 0 then exit;
    Zveza := Sprejmi(StProcesorja, Zveza); { vremo sporoilo pro } 
  until false; 
end; {Pobarvaj}

{-----------------------------------------------------------------------------}
{ Preostanek programa skrbi za simuliranje soasnega delovanja vseh           }
{ procesorjev.                                                                }
{-----------------------------------------------------------------------------}

const Sirina = 75; Visina = 22;
      StProcesorjev = Sirina * Visina;
      MaxVrsta = 8;
type
  PikselT = record
    Barva: integer;
    Sporocila: array[1..MaxVrsta] of integer;
    OdKod: array[1..MaxVrsta] of integer;
    { Sporocila[1..StSporocil - StPrihodnjih] so na voljo
      e zdaj, ostala pa ele v naslednji asovni enoti. }
    StSporocil: integer;
    StPrihodnjih: integer;
    { Prek naslednje spremenljivke si lahko podprogram
      SimulirajProcesor zapomni, do kod je pri izvajanju svojih
      ukazov e priel.  e bi ti podprogrami res tekli vsak
      na svojem procesorju, takega polja ne bi potrebovali, saj
      bi imel vsak procesor svoj programski tevec in po potrebi
      e kakno lokalno spremenljivko. }
    Stanje: integer; 
  end; {PikselT}
var P: array[0..StProcesorjev - 1] of PikselT;

function Vzorec(x, y: integer): integer; 
begin
  Assert((0 <= x) and (0 <= y) and (x < XMax) and (y < YMax));
  Vzorec := Ord('a') + (x + y) mod 3;
end; {Vzorec}

function JeSporocilo(StProcesorja: integer): integer; 
begin
  with P[StProcesorja] do begin
    if StSporocil - StPrihodnjih <= 0 then
      JeSporocilo := 0
    else
      JeSporocilo := OdKod[1];
  end; {with}
end; {JeSporocilo}

{ Vrne prvo sporoilo, ki je prilo po zvezi Zveza.
  Naloga pravi, da, e takega ni, podprogram aka na
  sporoilo, vendar bi se nam na simulator zaradi tega
  zakompliciral, zato te monosti ne podpiramo.  Podprogram
  SimulirajProcesor tako ali tako najprej poizve, na kateri
  zvezi je res sporoilo, preden klie Sprejmi. }
function Sprejmi(StProcesorja, Zveza: integer): integer; 
var i: integer;
begin
  with P[StProcesorja] do begin
    { Poiimo prvo sporoilo, prispelo po dani zvezi. }
    i := 1;
    while i <= StSporocil - StPrihodnjih do 
      if OdKod[i] = Zveza then break else i := i + 1;
    Assert(i <= StSporocil - StPrihodnjih);
    Sprejmi := Sporocila[i];
    { Zdaj ga pobriemo in nadaljnja sporoila premaknemo
      za eno mesto nazaj po tabeli. }
    while i < StSporocil do begin
      Sporocila[i] := Sporocila[i + 1];
      OdKod[i] := OdKod[i + 1];
      i := i + 1;
    end; {while}
    StSporocil := StSporocil - 1;
  end; {with}
end;

{ Doda sporoilo v vrsto prejemnika (X, Y),
  kot da bi prilo po dani zvezi. }
procedure DodajSporocilo(X, Y, Zveza, Sporocilo: integer);
begin
  with P[Y * Sirina + X] do begin
    Assert(StSporocil < MaxVrsta);
    StSporocil := StSporocil + 1;
    StPrihodnjih := StPrihodnjih + 1;
    Sporocila[StSporocil] := Sporocilo;
    OdKod[StSporocil] := Zveza;
  end; {with}
end; {DodajSporocilo}

procedure Poslji(StProcesorja, Zveza, Sporocilo: integer); 
const 
  DX: array[1..4] of integer = (0, 1, 0, -1);
  DY: array[1..4] of integer = (-1, 0, 1, 0);
  Nasprotna: array[1..4] of integer = (3, 4, 1, 2);
var X, Y: integer;
begin
  { Izraunajmo koordinati poiljatelja. }
  X := StProcesorja mod Sirina; Y := StProcesorja div Sirina;
  { Izraunajmo koordinati prejemnika. }
  X := X + DX[Zveza]; Y := Y + DY[Zveza];
  { Mogoe bo sporoilo padlo ez rob zaslona. }
  if (X < 0) or (Y < 0) or (X >= Sirina) or (Y >= Visina) then exit;
  { Dodajmo to sporoilo v prejemnikovo vrsto. }
  DodajSporocilo(X, Y, Nasprotna[Zveza], Sporocilo);
end; {Poslji}

function MojaBarva(StProcesorja: integer): integer; 
begin
  MojaBarva := P[StProcesorja].Barva;
end; {MojaBarva}

procedure PobarvajMe(StProcesorja, Barva: integer); 
var X, Y: integer;
begin
  X := StProcesorja mod Sirina; Y := StProcesorja div Sirina;
  P[StProcesorja].Barva := Barva;
  GotoXY(X + 1, Y + 1); Write(Chr(Barva));
end; {PobarvajMe}

{ Narie vse piksle na zaslon. }
procedure NarisiVse;
var S: string; X, Y: integer;
begin
  ClrScr; 
  for Y := 0 to Visina - 1 do begin
    S := '';
    for X := 0 to Sirina - 1 do 
      S := S + Chr(P[Y * Sirina + X].Barva);
    WriteLn(S);
  end; {for y}
end; {NarisiVse}

{ Postavi le vrednosti P[...].Barva, na zaslonu ne rie niesar. }
procedure PobarvajPravokotnik(xOd, yOd, xDo, yDo: integer);
const Sara = ',.;:';
var X, Y: integer;
begin
  if xDo < xOd then begin X := xOd; xOd := xDo; xDo := X; end;
  if yDo < yOd then begin Y := yOd; yOd := yDo; yDo := Y; end;
  for Y := yOd to yDo do for X := xOd to xDo do 
    P[Y * Sirina + X].Barva := Ord(Sara[1 + Random(Length(Sara))]);
end; {PobarvajPravokotnik}

var i: integer;
begin {Poplavljanje}

  { Za zaetek postavimo vse piksle na mejno barvo. }
  for i := 0 to StProcesorjev - 1 do with P[i] do begin
    Barva := MejnaBarva;
    StSporocil := 0; StPrihodnjih := 0;
    Stanje := 0;
  end; {with}
  { Nariimo nekaj pravokotnikov v nakljunih drugih barvah. }
  Randomize;
  for i := 1 to 5 do 
    PobarvajPravokotnik(Random(Sirina), Random(Visina), Random(Sirina), Random(Visina));

  { Izberimo neko zaetno toko. }
  while true do begin
    i := Random(StProcesorjev) + 1;
    if P[i].Barva <> MejnaBarva then break;
  end; {while}
  { Poljimo zaetni toki njeni koordinati. }
  DodajSporocilo(i mod Sirina, i div Sirina, 99, i mod Sirina);
  DodajSporocilo(i mod Sirina, i div Sirina, 99, i div Sirina);

  { Simulirajmo dogajanje. }
  NarisiVse;
  while not KeyPressed do begin
    Delay(100);
    for i := 0 to StProcesorjev - 1 do 
      P[i].StPrihodnjih := 0;
    for i := 0 to StProcesorjev - 1 do 
      SimulirajProcesor(i, P[i].Stanje);
  end; {with}
  
end. {Poplavljanje}

