program VodenjeProjektov; { RTK 1992.3.2 }
type
  PodmodulP = ^PodModulT;
  ModulP = ^ModulT;

  PodmodulT = record
    Podmodul: ModulP;
    Nasl: PodmodulP;
  end; {PodmodulT}

  ModulT = record
    Ime: string;
    Podmoduli: PodmodulP;
    NaslT: ModulP; { naslednji v razpreni tabeli }
    NaslS: ModulP; { naslednji v seznamu vseh modulov, ki jih je treba prevesti }
    Barva: (Bela, Siva, Crna);
  end; {ModulT}

const StRez = 10037;
var Reze: array[0..StRez - 1] of ModulP;

{ Pripravi prazno razpreno tabelo. }
procedure InitReze;
var i: integer;
begin
  for i := 0 to StRez - 1 do Reze[i] := nil;
end; {InitReze}

{ V razpreni tabeli Reze poie zapis, ki se nanaa
  na modul z danim imenom.  e takega e ni, ga doda v tabelo. }
function Modul(Ime: string): ModulP;
var i, hc: integer; P: ModulP;
begin
  { Izraunajmo razprilno kodo tega imena. }
  hc := 0;
  for i := 1 to Length(Ime) do
    hc := ((hc * 7) + Ord(Ime[i]) * i) mod StRez;
  { Ta modul se mora v razpreni tabeli nahajati
    v verigi Reze[hc].  Poglejmo, e je e tam. }
  P := Reze[hc];
  while P <> nil do 
    if P^.Ime = Ime then 
      begin Modul := P; exit; end
    else P := P^.NaslT;
  { Dodajmo ta modul v razpreno tabelo. }
  New(P); P^.Ime := Ime; P^.Podmoduli := nil;
  P^.NaslT := Reze[hc]; Reze[hc] := P;
  P^.NaslS := nil; P^.Barva := Bela;
  Modul := P;     
end; {Modul}

{ Doda modul z imenom ImePodmodula med podmodule modula Nadmodul. }
procedure DodajPodmodul(Nadmodul: ModulP; ImePodmodula: string);
var P: PodmodulP;
begin
  New(P); P^.Nasl := Nadmodul^.Podmoduli;
  P^.Podmodul := Modul(ImePodmodula);
  Nadmodul^.Podmoduli := P;
end; {DodajPodmodul}

{ Pobrie vse zapise tipa ModulT in PodmodulT, do katerih
  se da priti prek tabele Reze. }
procedure Pospravi;
var P: ModulP; R: PodmodulP; i: integer;
begin
  { Pobriimo vse module. }
  for i := 0 to StRez - 1 do while Reze[i] <> nil do begin
    { Pobriimo vse podmodule modula Reze[i]. }
    while Reze[i]^.Podmoduli <> nil do begin
      R := Reze[i]^.Podmoduli^.Nasl;
      Dispose(Reze[i]^.Podmoduli);
      Reze[i]^.Podmoduli := R;
    end; {while}
    { Pobriimo modul Reze[i]. }
    P := Reze[i]^.NaslT; Dispose(Reze[i]); Reze[i] := P;
  end; {for i, while}
end; {Pospravi}

{ Doda v seznam (Glava, Rep) modul M, e prej pa
  z rekurzivnimi klici tudi vse njegove podmodule. 
  V seznamu so moduli povezani prek kazalcev NaslS. }
procedure VrstniRed2(M: ModulP; var Glava, Rep: ModulP);
var
  R: PodmodulP; P: ModulP;
begin
  Assert(M^.Barva = Bela);
  { Oznaimo M kot sivega, kar pomeni, da trenutno
    pregledujemo njegove podmodule. }
  M^.Barva := Siva;
  { Preden se lahko prevede M, se morajo prevesti tudi
    njegovi podmoduli. }
  R := M^.Podmoduli;
  while R <> nil do begin
    P := R^.Podmodul;
    if P^.Barva = Siva then 
      WriteLn('Napaka: modula ', M^.Ime, ' in ', P^.Ime, ' sta odvisna drug od drugega.')
    else if P^.Barva = Bela then
      VrstniRed2(P, Glava, Rep);
    R := R^.Nasl;
  end; {while}
  { Dodajmo modul M na konec seznama. }
  if Rep = nil then Glava := M
  else Rep^.NaslS := M;
  Rep := M; Rep^.NaslS := nil;
  { Oznaimo M kot rnega, kar pomeni, da smo ga e dodali v seznam. }
  M^.Barva := Crna;
end; {VrstniRed2}

procedure VrstniRed(ImeGlavnegaModula: string);
var i: integer; P, Glava, Rep: ModulP;
begin
  { Oznaimo vse module kot bele, torej nepregledane. 
    Kasneje jih bo podprogram VrstniRed2 spreminjal
    v sive in rne. }
  for i := 0 to StRez - 1 do begin
    P := Reze[i]; 
    while P <> nil do 
      begin P^.Barva := Bela; P := P^.NaslT; end;
  end; {for i}
  { Zanimo pri glavnem modulu. }
  P := Modul(ImeGlavnegaModula);
  Glava := nil; Rep := nil; { seznam je sprva prazen }
  VrstniRed2(P, Glava, Rep);
  { Izpiimo dobljeni vrstni red. }
  WriteLn('Vrstni red prevajanja za glavni modul ', ImeGlavnegaModula, ': ');
  P := Glava;
  while P <> nil do 
    begin Write('  ', P^.Ime); P := P^.NaslS; end;
  WriteLn;
end; {VrstniRed}

function ObdelajVrstico(S: string): ModulP;
var i, n: integer;

  function Naslednji: string;
  var j: integer;
  begin
    while i <= n do if S[i] = ' ' then i := i + 1 else break;
    j := i;
    while j <= n do if S[j] = ' ' then break else j := j + 1;
    Naslednji := Copy(S, i, j - i); i := j;
  end; {Naslednji}

var Ime: string; P: ModulP;
begin
  ObdelajVrstico := nil;
  i := 1; n := Length(S);
  Ime := Naslednji; if Ime = '' then exit;
  P := Modul(Ime); ObdelajVrstico := P;
  Ime := Naslednji;
  repeat
    DodajPodmodul(P, Ime);
    Ime := Naslednji;
  until Ime = '';
end; {PreberiVhodnePodatke}

var S: string; P, Prvi: ModulP;
begin {VodenjeProjektov}
  InitReze;
  
  { Preberimo vhodne podatke. 
    V vsaki vrstici naj bi bilo ime nekega modula in nato
    imena njegovih podmodulov, loena s presledki. }
  Prvi := nil;
  while not Eof do begin
    ReadLn(S);
    P := ObdelajVrstico(S);
    if (Prvi = nil) and (P <> nil) then Prvi := P;
  end; {while}

  { Recimo, da je glavni modul tisti iz prve vrstice datoteke. }  
  VrstniRed(Prvi^.Ime);
  
  Pospravi;
end. {VodenjeProjektov}
  
  