program SimulatorMulticastinga; { RTK 1995.2.3 }
uses Crt;

const MaxIzhodov = 10;
var StIzhodov: integer;
function Beri(var ch: char): boolean; forward;
function Pisi(iz: integer; ch: char): boolean; forward;
{ Spodnjega podprograma naloga ne omenja.  Dodali smo ga
  zato, da ga obasno klie tekmovalev podprogram in lahko 
  na simulator takrat kaj izpie na zaslon in podobno. 
  e hoe uporabnik prekiniti simulacijo, vrne Zadremlji false. }
function Zadremlji: boolean; forward;

{ Tole bi morala biti lokalna spremenljivka podprograma Multicasting.
  Tukaj zunaj je le zato, da jo lahko Zadremlji uporabi pri izpisu statistike. }
var Dolz:       { za vsak izhodni kanal dolina vrste (t. akajoih znakov) }
    array[1..MaxIzhodov] of integer; 

{ To je tekmovalev podprogram. }
procedure Multicasting; 
const 
  VrstaM = 30;             { najveje mono tevilo znakov v akalni vrsti } 
var 
  Vrsta: array[1..VrstaM] of char; { kroni izravnalni pomnilnik }
  Rep: integer;            { kazalec na prvo prazno mesto v tabeli Vrsta } 
  Glava:                   { za vsak izhodni kanal svoj kazalec na rep vrste } 
    array[1..MaxIzhodov] of integer; 
  MaxDolz: integer;        { dolina najdalje vrste: max(Dolz[*]) } 
  Prebrano: integer;       { tevilo pravkar prebranih znakov } 
  iz: integer;             { tevilka izhodnega kanala }
  Konec: boolean; 
x: integer;
begin
  Rep := 1;
  for iz := 1 to StIzhodov do begin Glava[iz] := 1; Dolz[iz] := 0; end; 
  while Zadremlji do begin 
  
    { Ugotovi tevilo znakov v spominu (dolino najdalje vrste). } 
    MaxDolz := Dolz[1]; 
    for iz := 2 to StIzhodov do 
    if Dolz[iz] > MaxDolz then MaxDolz := Dolz[iz]; 
  
    { Beri, dokler ne zmanjka znakov ali prostora v pomnilniku. } 
    Konec := false; Prebrano := 0; 
    while (MaxDolz + Prebrano < VrstaM) and not Konec do 
      if not Beri(Vrsta[Rep]) then Konec := true
      else begin Prebrano := Prebrano + 1; 
                 Rep := (Rep mod VrstaM) + 1; end; 
    
    { Pii na vsak izhod, dokler se ne zatakne ali ne zmanjka znakov. } 
    for iz := 1 to StIzhodov do begin 
      Dolz[iz] := Dolz[iz] + Prebrano; 
      Konec := false; x := 0;
      while (Dolz[iz] > 0) and not Konec do 
        if not Pisi(iz, Vrsta[Glava[iz]]) then Konec := true
        else begin Dolz[iz] := Dolz[iz] - 1; x := x + 1;
                   Glava[iz] := (Glava[iz] mod VrstaM) + 1; end; 
    end; {for}
    
  end; {while}
end; {Multicasting} 

{-----------------------------------------------------------------------------}
{ Preostanek programa simulira dogajanje v omreju in prikazuje potek         }
{ simulacije na zaslonu.                                                      }
{-----------------------------------------------------------------------------}

{ Ta funkcija si izmiljuje nakljune ase med dvema
  zaporednima dogodkoma in to tako, da bo v povpreju
  doseeno zahtevano tevilo dogodkov na sekundo.
  Glej tudi rtk97s1n4.pas. }
function CasMedDogodkomaMs(DogodkovNaSekundo: real): integer;
var i: integer; 
begin
  i := Round(-1000.0 * Ln(1.0 - Random) / DogodkovNaSekundo);
  if i <= 0 then i := 1;
  CasMedDogodkomaMs := i;
end; {CasMedDogodkomaMs}

type 
  LinijaT = record
    ZnakovNaSekundo: real;
    Naslednji: longint; { kdaj bo lahko sprejet/poslan naslednji znak }
    StDoslej: integer;  { koliko jih je bilo sprejetih/poslanih doslej }
    KontrolnaVsota: integer; { vsota i * (i-ti sprejeti/poslani znak) po vseh i }
    Zaprta: boolean;    { ali je linija isto zaprta? }
  end; {LinijaT}

var Linije: array[0..MaxIzhodov] of LinijaT;
    Cas, CasZadnjihStatistik: longint;

function Beri(var ch: char): boolean;
begin
  with Linije[0] do begin
    if Zaprta or (Cas < Naslednji) then 
      begin Beri := false; exit; end;
    ch := Chr(Ord('a') + StDoslej mod 26);
    KontrolnaVsota := KontrolnaVsota + StDoslej * Ord(ch);
    Naslednji := Naslednji + CasMedDogodkomaMs(ZnakovNaSekundo);
    if Naslednji <= Cas - 1000 then Naslednji := Cas + 10;
    Inc(StDoslej); Beri := true;
  end; {with}
end;

function Pisi(iz: integer; ch: char): boolean;
begin
  with Linije[iz] do begin
    if Zaprta or (Cas < Naslednji) then 
      begin Pisi := false; exit; end;
    KontrolnaVsota := KontrolnaVsota + StDoslej * Ord(ch);
    Naslednji := Cas + CasMedDogodkomaMs(ZnakovNaSekundo);
    Inc(StDoslej); Pisi := true;
  end; {with}
end;

procedure PostaviLinijo(var L: LinijaT; ZnakovNaSekundo: real);
begin
  L.ZnakovNaSekundo := ZnakovNaSekundo;
  L.Naslednji := CasMedDogodkomaMs(ZnakovNaSekundo);
  L.StDoslej := 0;
  L.KontrolnaVsota := 0;
  L.Zaprta := false;
end; {PostaviLinijo}

function Zadremlji: boolean;
var i, StariCas: integer; c: char;
begin
  { Premaknimo se naprej po asu do naslednjega trenutka, 
    ko bo lahko kakna linija kaj sprejela ali oddala. }
  StariCas := Cas;
  for i := 0 to StIzhodov do with Linije[i] do 
    if (not Zaprta) and (Cas > Naslednji) and (Naslednji > StariCas) then Cas := Naslednji;
  if Cas = StariCas then { vse linije so zaprte }
    Cas := Cas + 100; 
  
  { Poglejmo, e je uporabnik pritisnil kakno tipko. }
  if KeyPressed then begin
    c := UpCase(ReadKey); if c = #0 then c := ReadKey;
    if (c = 'Q') or (c = 'X') then begin Zadremlji := false; exit; end;
    if (c = 'P') then begin
      Write('(pavza)', #13);
      c := UpCase(ReadKey); if c = #0 then c := ReadKey;
    end; {if}
    if (c >= '0') and (c <= Chr(Ord('0') + StIzhodov)) then
      with Linije[Ord(c) - Ord('0')] do Zaprta := not Zaprta;
  end; {if}

  { Obasno izpiimo statistike. }  
  if Cas > CasZadnjihStatistik + 300 then begin
    Write((Cas / 1000.0):7:3, ' '); 
    for i := 0 to StIzhodov do with Linije[i] do begin
      Write('[', i, ': ');
      if Zaprta then Write('Z') else Write('-');
      Write(' ', StDoslej:4, ' ', (KontrolnaVsota mod 100):2);
      if i > 0 then Write(' ', Dolz[i]:2);
      Write(']');
    end; {for i}
    Write(' ', (1000.0 * Linije[0].StDoslej / Cas):5:3, ' znakov/s');
    Write(#13);
    CasZadnjihStatistik := Cas;
  end; {if}
  
  Delay((Cas - StariCas) div 5);
  Zadremlji := true;
end; {Zadremlji}

var i, j: integer; x: real;
begin {SimulatorMulticastinga}

  WriteLn('Simulator multicastinga');
  if ParamCount < 2 then begin
    WriteLn('Kot parametre podaj kapacitete linij v znakih na sekundo');
    WriteLn('(najprej vhodna, nato izhodne).');
    WriteLn('Primer:  rtk95s2n3 1.0 1.2 1.0 0.7');
    exit; 
  end; {if}

  StIzhodov := ParamCount - 1;
  for i := 0 to StIzhodov do begin
    Val(ParamStr(i + 1), x, j);
    PostaviLinijo(Linije[i], x);
  end; {for}

  WriteLn('Za vsako linijo pie [i: a b c d]');
  WriteLn('  i = tevilka linije (0 = vhodna, ostale izhodne)');
  WriteLn('  a = "Z", e je zaprta (pritiskaj na tipke "0", "1", "2", itd.), sicer "-"');
  WriteLn('  b, c = tevilo in kontrolna vsota doslej prenesenih znakov');
  { e npr. zapremo vhodno in odpremo izhodne linije ter poakamo,
    da se vrste izpraznijo, bi morale imeti vse linije enako tevilo
    prenesenih znakov in enako kontrolno vsoto. }
  WriteLn('  d = tevilo znakov v vrsti (samo pri izhodnih linijah)');
  Writeln('Na koncu je dosedanja povprena hitrost prenosa.');
  WriteLn;
  WriteLn('Pritisni Q ali X za izhod, P za pavzo, ');
  WriteLn('tevilske tipke za zapiranje/odpiranje linij.');
  WriteLn;
  Write('Kapacitete linij (znakov/s): ');
  for i := 0 to StIzhodov do Write('[', i, ': ', 
    Linije[i].ZnakovNaSekundo:4:1, ']');
  WriteLn;

  Cas := 0;
  Multicasting;
  
end. {SimulatorMulticastinga}
