program NajkrajsePotiVMrezi; { RTK 1995.2.4 }

const 
  MaxRac = 100;
  MaxSmeri = 5;

{ Vmesnik, ki ga vidi KamNajPosljem.  Ker se ne izvaja
  na ve raunalnikih, ampak na enem samem, mu bomo
  MojNaslov podali kar kot parameter, on pa ga mora
  podobno podati funkcijama Poslji in Sprejmi. }
type SporociloT = integer;  
function Poslji(MojNaslov, VSmer: integer; 
                Sporocilo: SporociloT): boolean; forward;
function Sprejmi(MojNaslov: integer; var IzSmeri: integer; 
                 var Sporocilo: SporociloT): boolean; forward;

type
  { Na enem raunalniku bomo morali simulirati delovanje 
    ve neodvisnih izvajanj podprograma KamNajPosljem.
    Zato se bo izvajal po kokih, namesto nekaterih lokalnih
    spremenljivk pa bo uporabljal primerek takne strukture. }
  KontekstT = record
    SmeriRacunalnikov: array[1..MaxRac] of integer; 
    SteviloPoznanihRacunalnikov: integer; 
    Neinicializiran: boolean; { glavni del programa ga postavi na true pred prvim klicem }
    Konec: boolean;           { KamNajPosljem ga postavi na true, ko ve e vse }
  end; {KontekstT}

var SteviloRacunalnikov: integer;

procedure KamNajPosljem(MojNaslov: integer; var Kontekst: KontekstT); 
var Stevec: integer; 
    VSmer, IzSmeri, SprejetiNaslov: integer;
begin
  with Kontekst do 
  
    if Neinicializiran then begin
      Neinicializiran := false; Konec := false;
      SteviloPoznanihRacunalnikov := 1; 
      for Stevec := 1 to SteviloRacunalnikov do 
        SmeriRacunalnikov[Stevec] := 0; 
      for VSmer := 1 to MaxSmeri do Poslji(MojNaslov, VSmer, MojNaslov); 
    end 
  
    else while Sprejmi(MojNaslov, IzSmeri, SprejetiNaslov) do begin
      if (SmeriRacunalnikov[SprejetiNaslov] = 0) and 
                   (SprejetiNaslov <> MojNaslov) then begin 
        SteviloPoznanihRacunalnikov := SteviloPoznanihRacunalnikov + 1; 
        SmeriRacunalnikov[SprejetiNaslov] := IzSmeri; 
        for VSmer := 1 to MaxSmeri do 
          if VSmer <> IzSmeri then Poslji(MojNaslov, VSmer, SprejetiNaslov); 
      end; {if}
      Konec := SteviloPoznanihRacunalnikov >= SteviloRacunalnikov;
    end; {if}

end; {KamNajPosljem} 

{-----------------------------------------------------------------------------}
{ Preostanek programa simulira dogajanje v mrei.                             }
{-----------------------------------------------------------------------------}

type
  VrstaP = ^VrstaT;
  VrstaT = record
    S: SporociloT;
    IzSmeri: integer;
    Nasl: VrstaP;
    KdajPride: integer;
  end; {VrstaT}
var
  Povezave: array[1..MaxRac, 1..MaxSmeri] of 0..MaxRac;
  Glave, Repi: array[1..MaxRac] of VrstaP;
  Kontekst: array[1..MaxRac] of KontekstT;
  Cas: integer;

function Poslji(MojNaslov, VSmer: integer; 
                Sporocilo: SporociloT): boolean; 
var Komu: integer; P: VrstaP;
begin
  { Ali ta smer sploh obstaja? }
  Komu := Povezave[MojNaslov, VSmer];
  if Komu = 0 then begin Poslji := false; exit; end;
  New(P); P^.S := Sporocilo; 
  { Ker na simulator simulira raunalnike po vrsti, bi
    se lahko zgodilo, da npr. raunalnik i polje sporoilo
    raunalniku i+1, ta pa ga prevzame e v okviru istega
    asovnega koraka.  To ni le nerealistino, pa pa obiajno
    tudi povzroi, da si ustvari raunalnik i+1 napano 
    podobo o tem, v kateri smeri gre najkraja pot do nekaterih
    drugih raunalnikov.  Zato uporabljamo polje KdajPride,
    da podprogram Sprejmi ne bo prezgodaj prevzemal sporoil. }
  P^.KdajPride := Cas + 1;
  { Iz katere svoje smeri bo prejemnik dobil to sporoilo? }
  P^.IzSmeri := 1;
  while Povezave[Komu, P^.IzSmeri] <> MojNaslov do Inc(P^.IzSmeri);
  { Dodajmo sporoilo na konec prejemnikove vrste. }
  if Repi[Komu] = nil then Glave[Komu] := P
  else Repi[Komu]^.Nasl := P;
  Repi[Komu] := P; P^.Nasl := nil; Poslji := true;
  {WriteLn('[T = ', Cas:4, ']  ', MojNaslov, ' poilja ', 
           Komu, ' sporoilo ', Sporocilo, '.');}
end; {Poslji}

function Sprejmi(MojNaslov: integer; var IzSmeri: integer; 
                 var Sporocilo: SporociloT): boolean;
var P: VrstaP; 
begin
  if Glave[MojNaslov] = nil then begin Sprejmi := false; exit; end;
  if Glave[MojNaslov]^.KdajPride > Cas then begin Sprejmi := false; exit; end;
  P := Glave[MojNaslov]; Glave[MojNaslov] := P^.Nasl;
  if P^.Nasl = nil then Repi[MojNaslov] := nil;
  IzSmeri := P^.IzSmeri; Sporocilo := P^.S;
  Dispose(P); Sprejmi := true;
  {WriteLn('[T = ', Cas:4, ']  ', MojNaslov, ' prejme od ', 
           Povezave[MojNaslov, IzSmeri], ' sporoilo ', Sporocilo, '.');}
end; {Sprejmi}

var SeKajTece: boolean; i, j, k: integer; T: text;
begin {NajkrajsePotiVMrezi}

  { Preberimo opis povezav. }
  if ParamCount >= 1 then Assign(T, ParamStr(1))
  else Assign(T, 'rtk95s2n4.in');
  Reset(T); ReadLn(T, SteviloRacunalnikov);
  for i := 1 to SteviloRacunalnikov do 
    for k := 1 to MaxSmeri do Povezave[i, k] := 0;
  while true do begin
    ReadLn(T, i, j); if (i <= 0) or (j <= 0) then break;
    k := 1; while Povezave[i, k] > 0 do Inc(k);
    Povezave[i, k] := j;
    k := 1; while Povezave[j, k] > 0 do Inc(k);
    Povezave[j, k] := i;
  end; {while}
  Close(T);
  WriteLn('Povezave:');
  for i := 1 to SteviloRacunalnikov do begin
    Write(i:2, ':');
    for k := 1 to MaxSmeri do Write(' ', Povezave[i, k]:2);
    WriteLn;
  end; {for i}
  
  { Pripravimo se na izvajanje podprograma KamNajPosljem. }
  for i := 1 to SteviloRacunalnikov do begin 
    Kontekst[i].Neinicializiran := true; 
    Kontekst[i].Konec := false; 
    Glave[i] := nil; Repi[i] := nil;
  end; {for}
  
  Cas := 0;
  repeat
    SeKajTece := false;
    for i := 1 to SteviloRacunalnikov do 
      if not Kontekst[i].Konec then begin
        KamNajPosljem(i, Kontekst[i]);
        SeKajTece := SeKajTece or not Kontekst[i].Konec;
      end; {if}
    { Izpiimo malo statistike. }
    Write('[T = ', Cas:4, '] ');
    for i := 1 to SteviloRacunalnikov do
      Write(' ', Kontekst[i].SteviloPoznanihRacunalnikov:2);
    WriteLn;
    Inc(Cas); 
  until not SeKajTece;

  { Izpiimo dobljene usmerjevalne tabele. }
  { Za vsak raunalnik dobimo vrstico parov j/s,
    ki povedo, da je treba sporoilo poslati sosedu s,
    e hoemo, da bi im hitreje prilo do raunalnika j. }
  for i := 1 to SteviloRacunalnikov do begin
    Write(i, ':');
    for j := 1 to SteviloRacunalnikov do
      if j = i then Write(' ', j, '/', i)
      else if Kontekst[i].SmeriRacunalnikov[j] = 0 then Write(' ', j, '/-')
      else Write(' ', j, '/', Povezave[i, Kontekst[i].SmeriRacunalnikov[j]]);
    WriteLn;
  end; {for i}
  
end. {NajkrajsePotiVMrezi}