program InternetniPredpomnilnik; { RTK 1997.3.4 }
uses Crt;

type
  TokP = ^TokT;
  TokT = record
    Tip: (Uporabnik, Predpomnilnik, Internet);
    { e je Tip = Uporabnik, gre za tok, po katerem
      naj bi na sistem poiljal podatke uporabniku. 
      Veljavni polji sta Naslov (za kateri naslov so ti 
      podatki) in Vsebina (kar smo doslej e poslali uporabniku).
      
      e je Tip = Predpomnilnik, gre za tok, po
      katerem naj bi tekmovalev del sistema poiljal
      podatke predpomnilniku, da jih bo ta shranil. 
      Veljavni polji sta Naslov in Vsebina (kot zgoraj). 
      
      e je Tip = Internet, gre za tok, po
      katerem naj bi prihajali iz interneta podatki
      do tekmovalevega dela sistema.  Poleg polj Naslov
      in Vsebina (vsebina celega internetnega vira s
      tega naslova) sta veljavni e ZePrejeto (koliko 
      podatkov smo doslej e dobili iz interneta) in
      NaslednjiPrejem (as, ko bomo dobili naslednji 
      koek podatkov). }
    Naslov: string;
    Vsebina: string; 
    ZePrejeto, NaslednjiPrejem: integer;
    { Tokovi so povezani v verigo. }
    Prej, Nasl: TokP;
    { Tole uporabljata podprograma Zahtevek in Prispelo.
      V prvotnem besedilu naloge je bilo zamiljeno, da
      namesto TokP dobita neko majhno pozitivno celo tevilo,
      pa bi potem uporabila kar tisto kot indeks v neko tabelo. }
    Indeks: integer;
  end; {TokT}

{-----------------------------------------------------------------------------}
{ Podprograma, ki ju je moral napisati tekmovalec.                            }
{-----------------------------------------------------------------------------}

{ Vmesnik, ki ga vidita tadva podprograma. }
function Zahtevaj(Naslov: string): TokP; forward;
procedure Poslji(Podatki: string; Tok: TokP); forward;
function Shranjen(Naslov: string; var Podatki: string): TokP; forward;

const MaxZahtev = 10000; { Najve dovoljenih istoasnih zahtev. } 
type DvaTokova = record DoUporabnika, VPomnilnik: TokP end;
var Zahteve: array[1..MaxZahtev] of DvaTokova; 

{ Ta podprogram kliemo, ko uporabnik polje nov zahtevek. }
procedure Zahtevek(Naslov: string; Tok: TokP);
var Podatki: string; 
    VmTok, NovTok: TokP; 
begin
  VmTok := Shranjen(Naslov, Podatki); 
  if VmTok = nil then begin
    Poslji(Podatki, Tok);
    Poslji('', Tok);
  end else begin 
    NovTok := Zahtevaj(Naslov); 
    { Pripiimo novemu toku nek indeks v tabelo Zahteve. }
    NovTok^.Indeks := 1;
    while Zahteve[NovTok^.Indeks].DoUporabnika <> nil do 
      Inc(NovTok^.Indeks);
    Zahteve[NovTok^.Indeks].DoUporabnika := Tok; 
    Zahteve[NovTok^.Indeks].VPomnilnik := VmTok; 
  end; {if}
end; {Zahtevek}

{ Ta podprogram kliemo, ko pride kaj novih podatkov iz interneta. }
procedure Prispelo(Podatki: string; Tok: TokP);
begin
  Poslji(Podatki, Zahteve[Tok^.Indeks].DoUporabnika); 
  Poslji(Podatki, Zahteve[Tok^.Indeks].VPomnilnik); 
  if Length(Podatki) = 0 then begin 
    Zahteve[Tok^.Indeks].DoUporabnika := nil; 
    Zahteve[Tok^.Indeks].VPomnilnik := nil; 
  end; {if}
end; {Prispelo}

{-----------------------------------------------------------------------------}
{ Preostanek programa simulira nastajanje uporabnikovih zahtev, prejemanje    }
{ podatkov iz interneta in delovanje internetnega medpomnilnika.              }
{-----------------------------------------------------------------------------}

var
  { Predpomnilnik je tu kar seznam struktur tipa TokT, v katerih 
    so e naslovi in vsi podatki. }
  Tokovi, Shranjeno: TokP;
  Cas: integer; 
  { Kdaj bo priel naslednji uporabnikov zahtevek? }
  NaslednjiZahtevek: integer;

{ Ustvari nov zapis tipa TokT in ga doda na zaetek verige Tokovi. }
function NovTok: TokP;
var P: TokP;
begin
  New(P); P^.Prej := nil; P^.Nasl := Tokovi; 
  if Tokovi <> nil then Tokovi^.Prej := P;
  Tokovi := P; NovTok := P;
end; {NovTok}

{ Zbrie P iz verige Tokovi; e je Unici = true, klie tudi Dispose. }
procedure ZapriTok(P: TokP; Unici: boolean);
begin
  if P^.Prej <> nil then P^.Prej^.Nasl := P^.Nasl
  else Tokovi := P^.Nasl;
  if P^.Nasl <> nil then P^.Nasl^.Prej := P^.Prej;
  P^.Prej := nil; P^.Nasl := nil;
  if Unici then Dispose(P);
end; {ZapriTok}

{ Izmisli si as med dvema prejemoma podatkov 
  za neko zahtevo iz interneta. }
function CasMedPrejemoma: integer;
  begin CasMedPrejemoma := Random(5) + 1; end;
  
{ Za pojasnilo formule v tem podprogramu glej rtk97s1n4.pas. }
function CasMedZahtevkoma: integer;
const ZahtevkovNaSekundo = 1.0 / 15.0;
var t: integer;
begin
  t := Round(-Ln(1.0 - Random) / ZahtevkovNaSekundo); 
  if t < 1 then t := 1; 
  CasMedZahtevkoma := t; 
end; {CasMedZahtevkoma}

{ Ustvari nov tok tipa Internet in si ob tem e 
  izmisli, kakne podatke naj bi ta tok dobil iz interneta. }
function Zahtevaj(Naslov: string): TokP; 
var P: TokP; i, L: integer;
begin
  P := NovTok;
  P^.Tip := Internet;
  P^.Naslov := Naslov;
  { Izmislimo si, kakni podatki tiijo na tem naslovu. 
    Ta del programa naj bo deterministien (e se vekrat
    zahteva isti naslov, naj pridejo vedno enaki podatki). }
  L := 0; for i := 1 to Length(Naslov) do Inc(L, Ord(Naslov[i]));
  L := 30 + (L mod 100);
  P^.Vsebina := '';
  while Length(P^.Vsebina) < L do P^.Vsebina := P^.Vsebina + Naslov;
  P^.Vsebina := Copy(P^.Vsebina, 1, L);
  { }
  P^.ZePrejeto := 0;
  P^.NaslednjiPrejem := Cas + CasMedPrejemoma;
  WriteLn('[T = ', Cas:4, ']  Zahtevamo "', Naslov, '" iz interneta, prilo bo ', L, ' znakov.');
  Zahtevaj := P;
end; {Zahtevaj}

{ Polje podatke na dani tok.  e je Length(Podatki) = 0,
  tok zaprej.  Tok mora biti tipa Uporabnik ali pa Predpomnilnik.
  Uporabniki tokovi se ob zaprtju uniijo, predpomnilniki
  pa se preselijo v seznam Shranjeno. }
procedure Poslji(Podatki: string; Tok: TokP); 
begin
  if Tok^.Tip = Uporabnik then begin
    { Tekmovalev del sistema poilja podatke uporabniku. }
    Tok^.Vsebina := Tok^.Vsebina + Podatki;
    if Length(Podatki) > 0 then 
      WriteLn('[T = ', Cas:4, ']  Uporabnik prejme ', Length(Podatki), 
              ' znakov (doslej skupaj ', Length(Tok^.Vsebina), ') za "', Tok^.Naslov, '".')
    else begin
      { Prili so vsi podatki -- tok zaprimo. }
      WriteLn('[T = ', Cas:4, ']  Uporabnik je prejel vseh ', Length(Tok^.Vsebina), 
              ' znakov za "', Tok^.Naslov, '".');
      WriteLn('            ', Tok^.Vsebina);
      ZapriTok(Tok, true);
    end; {if}
  end 
  
  else if Tok^.Tip = Predpomnilnik then begin
    { Tekmovalev del sistema poilja podatke predpomnilniku. }
    Tok^.Vsebina := Tok^.Vsebina + Podatki;
    if Length(Podatki) > 0 then 
      WriteLn('[T = ', Cas:4, ']  Predpomnilnik prejme ', 
              Length(Podatki), ' znakov (doslej skupaj ',
              Length(Tok^.Vsebina), ') za "', Tok^.Naslov, '".')
    else begin 
      { Prili so vsi podatki, torej tok zaprimo in 
        podatke preselimo v seznam 'Shranjeno'. }
      WriteLn('[T = ', Cas:4, ']  Predpomnilnik je prejel vseh ', 
              Length(Tok^.Vsebina), ' znakov za "', Tok^.Naslov, '".');
      WriteLn('            ', Tok^.Vsebina);
      ZapriTok(Tok, false);
      Tok^.Nasl := Shranjeno; Shranjeno := Tok;
    end;
  end;
end; {Poslji}

{ e je Naslov shranjen v predpomnilniku (seznam Shranjeno),
  vrne nil in njegove podatke v Podatki; sicer ustvari
  nov tok (tipa Predpomnilnik), kamor naj tekmovaleva koda
  polje podatke za ta naslov, ko jih bo prejela iz interneta. }
function Shranjen(Naslov: string; var Podatki: string): TokP;
var P: TokP;
begin
  P := Shranjeno;
  while P <> nil do begin
    if P^.Naslov = Naslov then begin 
      WriteLn('[T = ', Cas:4, ']  Shranjen("', Naslov, '"): podatki (', 
              Length(P^.Vsebina), ' znakov) so e v predpomnilniku.');
      Podatki := P^.Vsebina; Shranjen := nil; exit; 
    end; {if}
    P := P^.Nasl;
  end; {while}
  P := NovTok;
  P^.Tip := Predpomnilnik;
  P^.Naslov := Naslov;
  P^.Vsebina := '';
  Shranjen := P;
end; {Shranjen}

procedure IzpisiTokove;
var Tok: TokP;
begin
  Write('[T = ', Cas:4, ']  Tokovi: ');
  Tok := Tokovi;
  while Tok <> nil do begin
    case Tok^.Tip of
    Uporabnik: Write('User:');
    Internet: Write('Net:');
    Predpomnilnik: Write('Cache:');
    end; {Tok}
    Write(Tok^.Naslov, ':');
    if Tok^.Tip = Internet then Write(Tok^.ZePrejeto, '/', Length(Tok^.Vsebina))
    else Write(Length(Tok^.Vsebina));
    Write(' '); 
    Tok := Tok^.Nasl;
  end; {while}
  WriteLn;
end; {IzpisiTokove}

const 
  StNaslovov = 10;
  Naslovi: array[1..StNaslovov] of string = (
    'ABC', 'DEF', 'GHI', 'JKL', 'MNO', 
    'PQR', 'STU', 'VWX', 'YZA', 'CSZ');
var Tok, Nasl: TokP; Koliko: integer; c: char;
begin {InternetniPredpomnilnik}

  WriteLn('Simulator internetnega predpomnilnika');
  WriteLn('Pritisni Q ali X za konec, poljubno drugo tipko za pavzo.');
  Cas := 0; Tokovi := nil; Shranjeno := nil;
  NaslednjiZahtevek := CasMedZahtevkoma;
  
  while true do begin
  
    if Cas >= NaslednjiZahtevek then begin
      { Izmislimo si nov zahtevek, ki da je priel od uporabnika. }
      Tok := NovTok; Tok^.Tip := Uporabnik;
      Tok^.Vsebina := ''; Tok^.Naslov := Naslovi[1 + Random(StNaslovov)];
      WriteLn('[T = ', Cas:4, ']  Uporabnik polje zahtevek za "', Tok^.Naslov, '".');
      Zahtevek(Tok^.Naslov, Tok);
      NaslednjiZahtevek := Cas + CasMedZahtevkoma;
      IzpisiTokove;
    end; {if}
    
    Tok := Tokovi;
    while Tok <> nil do begin
      if (Tok^.Tip = Internet) and (Tok^.NaslednjiPrejem <= Cas) then begin

        { Obvestimo tekmovalevo kodo, da je prilo nekaj novih podatkov. }
        Koliko := Random(20) + 5; { izmislimo si, koliko podatkov je prilo }
        if Tok^.ZePrejeto + Koliko > Length(Tok^.Vsebina) then
          Koliko := Length(Tok^.Vsebina) - Tok^.ZePrejeto;
        WriteLn('[T = ', Cas:4, ']  Iz interneta je prilo e ', Koliko, 
                ' znakov za "', Tok^.Naslov, '" (doslej skupaj ', 
                Tok^.ZePrejeto + Koliko, ' od ', Length(Tok^.Vsebina), ').');
        Prispelo(Copy(Tok^.Vsebina, Tok^.ZePrejeto, Koliko), Tok);
        Inc(Tok^.ZePrejeto, Koliko);
        { Izmislimo si, kdaj bodo prili naslednji podatki. }
        Tok^.NaslednjiPrejem := Cas + CasMedPrejemoma;
        { e so prili e vsi podatki, tok zaprimo. }
        if Koliko = 0 then begin
          Nasl := Tok^.Nasl; ZapriTok(Tok, true); Tok := Nasl; 
          IzpisiTokove; continue;
        end;

      end; {if}
      Tok := Tok^.Nasl;
    end; {Tok}
    
    if KeyPressed then begin
      c := UpCase(ReadKey); if c = #0 then c := ReadKey;
      if (c = 'Q') or (c = 'X') then break;
      Write('(pavza)', #13);
      c := UpCase(ReadKey); if c = #0 then c := ReadKey;
      if (c = 'Q') or (c = 'X') then break;
    end; {if}
    
    Delay(100); Inc(Cas);
    
  end; {while}
  
  { Poistimo tokove. }
  while Shranjeno <> nil do begin
    Nasl := Shranjeno^.Nasl;
    ZapriTok(Shranjeno, true);
    Shranjeno := Nasl;
  end; {while}
  while Tokovi <> nil do begin
    Nasl := Tokovi^.Nasl;
    ZapriTok(Tokovi, true);
    Tokovi := Nasl;
  end; {while}
  
end. {InternetniPredpomnilnik}
