program KvadratEnic;
{ Primerjava asa izvajanja na matriki 100 * 100 samih enic:
    NajvecjiKvadratEnic1   0.23 s
    NajvecjiKvadratEnic2   0.015 s
  Na matrikah, nakljuno zapolnjenih z nilami in enicami,
  pa ni v asu izvajanja praktino nobene razlike. }

const MaxM = 100; MaxN = 100;
type Matrika = array[1..MaxM, 1..MaxN] of integer; 

{ Naivna reitev, ki za vsako polje gleda veje in veje 
  kvadrate s spodnjim desnim kotom na tistem polju.
  asovna zahtevnost je tako O(m n (min(m, n))^2). }
function NajvecjiKvadratEnic1(var a: Matrika; m, n: integer): integer; 
var i, j, k, s, sm: integer; ok: boolean; 
begin
  sm := 0; { sm je stranica najvejega doslej najdenega kvadrata } 
  for i := 1 to m do for j := 1 to n do begin 
    { Poiskali bomo najveji kvadrat enic s spodnjim desnim 
      kotom na polju a[i, j]. Njegova stranica bo s.}
    s := 0; ok := true; 
    while (s < i) and (s < j) and ok do begin 
      { Vemo, da obstaja primeren kvadrat s stranico s. 
        Ali obstaja tudi tak s stranico s + 1? }
      k := 1; s := s + 1; 
      while (k <= s) and ok do begin   
        if a[i - s + 1, j - k + 1] = 0 then ok := false; 
        if a[i - k + 1, j - s + 1] = 0 then ok := false;  
        k := k + 1; 
      end; {while k} 
    end; {while s} 
    { e se je zanka ustavila, ker smo pri poskusu poveevanja 
      kvadrata odkrili nilo, je bilo zadnje poveanje s-ja 
      neupravieno. }
    if not ok then s := s - 1; 
    { e je to nov najveji kvadrat, si ga zapomnimo. } 
    if sm < s then sm := s; 
  end; {for i, j} 
  NajvecjiKvadratEnic1 := sm; 
end; {NajvecjiKvadratEnic1}

function Min(a, b: integer): integer;
  begin if a < b then Min := a else Min := b; end;

{ Bolja reitev, ki si za vsako polje zapomni stranico 
  najvejega kvadrata enic s spodnjim desnim kotom v tem
  polju; ta podatek pride prav pri iskanju kvadratov v
  spodnjem in desnem sosedu tega polja.  asovna zahtevnost
  je le O(m n). }
function NajvecjiKvadratEnic2(a: Matrika; m, n: integer): integer; 
var s: array[0..MaxN] of integer; i, j, sm, sp, sn: integer; 
begin
  { Tabela s se na zaetku nanaa na i = 0, kar lei  
    zunaj tabele in si zato mislimo tam same nile. }
  sm := 0; for j := 0 to n do s[j] := 0; 
  for i := 1 to m do begin 
    sp := 0; 
    for j := 1 to n do begin 
      { V polju s[j - 1] je zdaj vrednost s(i, j - 1); 
        v polju s[j] je vrednost s(i - 1, j); 
        vrednost s(i - 1, j - 1) pa imamo zaasno v sp. 
        Izraunajmo s(i, j) in jo zaasno shranimo v sn.}
      if a[i, j] = 0 then sn := 0 
      else sn := 1 + Min(Min(s[j], s[j - 1]), sp); 
      if sn > sm then sm := sn; { Nov najveji kvadrat. } 
      { V naslednji iteraciji bomo potrebovali vrednost s(i - 1, j), 
        ki je trenutno e v s[j]; zapomnimo si jo v sp, da bomo 
        lahko zdaj v s[j] vpisali pravkar izraunano vrednost s(i, j), 
        ki jo trenutno hranimo v sn. }
      sp := s[j]; s[j] := sn; 
    end; {for j} 
  end; {for i} 
  NajvecjiKvadratEnic2 := sm; 
end; {NajvecjiKvadratEnic2} 

var T: text; m, n, x, y, r, Alg: integer; a: Matrika; c: char;
begin {KvadratEnic}
  { Kateri algoritem naj uporabimo? }
  if ParamCount >= 1 then Val(ParamStr(1), Alg, x)
  else Alg := 1;
  { Odprimo vhodno datoteko. }
  if ParamCount >= 2 then Assign(T, ParamStr(2))
  else Assign(T, 'rtk99s2n2.in');
  Reset(T);
  while not Eof(T) do begin
    { Preberimo naslednji testni primer. }
    ReadLn(T, m, n);
    if (m = 0) or (n = 0) then break;
    for y := 1 to m do begin
      x := 1;
      while x <= n do begin
        Read(T, c);
        if (c = '1') or (c = '#') then a[y, x] := 1
        else if (c = '0') or (c = '.') then a[y, x] := 0
        else if (c = ' ') or (c = #9) then continue
        else begin WriteLn('Napaka v vhodnih podatkih!'); Close(T); exit; end;
        x := x + 1;
      end; {while}
      ReadLn(T);
    end; {for}
    { Izpiimo rezultat. }
    if Alg = 1 then r := NajvecjiKvadratEnic1(a, m, n)
    else r := NajvecjiKvadratEnic2(a, m, n);
    WriteLn('Matrika ', m, ' * ', n, ', algoritem ', Alg, ': odgovor je ', r, '.');
  end; {while}
  Close(T);
end. {KvadratEnic}
