program Zaokrozanje; { RTK 2000.1.2 }
{ Sintaksa: 
  rtk00s1n2 [N]       -- vpraa za x, izpie ZaokroziN(x)
  rtk00s1n2 N x       -- izpie ZaokroziN(x)
  rtk00s1n2 N a b [M] -- klie ZaokroziN(x) za vse x iz a <= x <= b.
                         To ponovi M-krat in meri as (privzeta vrednost M = 1). }

uses Windows;
var ni: int64;

{$DEFINE _STEJ_ITERACIJE}

{ Osnovna razliica: po ena iteracija za vsak bit v x,
  do najvijega priganega. }
function Zaokrozi1(x: integer): integer; 
var xx, y: integer; 
begin
  if x <= 0 then y := 0 else y := 1; 
  xx := x; { Zapomnimo si prvotno vrednost x. }
  while x > 1 do begin 
    { Lahko bi uporabili shr in shl, a pustimo to prevajalniku. } 
    x := x div 2; y := y * 2;  
{$IFDEF STEJ_ITERACIJE}
    Inc(ni);
{$ENDIF}
  end; {while}
  { Zdaj je y tak kot xx, v katerem smo ugasnili 
    vse enice razen najvije. }
  if xx > y then y := y * 2; 
  Zaokrozi1 := y; 
end; {Zaokrozi1}  

{ Kot Zaokrozi1, le da uporablja shl in shr. }
function Zaokrozi1a(x: integer): integer; 
var xx, y: integer; 
begin
  if x <= 0 then y := 0 else y := 1; 
  xx := x; { Zapomnimo si prvotno vrednost x. }
  while x > 1 do begin 
    x := x shr 1; y := y shl 1;  
{$IFDEF STEJ_ITERACIJE}
    Inc(ni);
{$ENDIF}
  end; {while}
  { Zdaj je y tak kot xx, v katerem smo ugasnili 
    vse enice razen najvije. }
  if xx > y then y := y shl 1; 
  Zaokrozi1a := y; 
end; {Zaokrozi1a}  

function Zaokrozi1b(x: integer): integer; 
var y: integer; 
begin
  if x <= 0 then y := 0 else y := 1; 
  while x > y do begin
    y := y shl 1;
{$IFDEF STEJ_ITERACIJE}
    Inc(ni);
{$ENDIF}
  end; {while}
  Zaokrozi1b := y; 
end; {Zaokrozi1b}  

{ Izboljana razliica: po ena iteracija za vsak prigan bit v x. }
function Zaokrozi2(x: integer): integer; 
var xx, y: integer; 
begin
  xx := x; if x <= 0 then y := 0; 
  while x > 0 do begin 
    y := x; 
    x := x and (x - 1); { Ugasnimo najnijo prigano enico. }
{$IFDEF STEJ_ITERACIJE}
    Inc(ni);
{$ENDIF}
  end; {while}
  { Zdaj je y tak kot xx, v katerem smo ugasnili 
    vse enice razen najvije. }
  if xx > y then y := y * 2; 
  Zaokrozi2 := y; 
end; {Zaokrozi2} 

{ Izboljana razliica: po ena iteracija za vsako strnjeno
  zaporedje priganih bitov v x. }
function Zaokrozi3(x: integer): integer; 
var xx, y: integer; 
begin
  xx := x; if x <= 0 then y := 0; 
  { Naslednji stavek pusti prigano le najvijo enico
    od vsake strnjene skupine enic. }
  x := x and not (x div 2);
  while x > 0 do begin 
    y := x; 
    x := x and (x - 1); { Ugasnimo najnijo prigano enico. }
{$IFDEF STEJ_ITERACIJE}
    Inc(ni);
{$ENDIF}
  end; {while}
  { Zdaj je y tak kot xx, v katerem smo ugasnili 
    vse enice razen najvije. }
  if xx > y then y := y * 2; 
  Zaokrozi3 := y; 
end; {Zaokrozi3} 

{ Ta funkcija se zanaa na to, da je tip double predstavljen
  po standardu IEEE 754 in da imamo na voljo tip int64 za
  64-bitna cela tevila. }
function Zaokrozi4(x: integer): integer;
var xx: double; y: integer;
begin
  if x <= 0 then y := 1
  else begin
    xx := x; { Pretvorimo x v 64-bitno tevilo s plavajoo vejico. }
    y := (integer(int64(xx) shr 52) and 2047) - 1023; { Izluimo eksponent. }
    { Na tem mestu vemo, da je 2^y <= x < 2^(y+1). }
    y := 1 shl y; 
    if y <> x then y := y shl 1;
  end; {if}
  Zaokrozi4 := y;
end; {Zaokrozi4}

function Zaokrozi5(x: integer): integer;
begin
  { Oznaimo zaetno vrednost spremenljivke x z x0. 
    Naj bo 2^(k-1) < x0 <= 2^k.  Naa funkcija mora
    torej vrniti 2^k. }
  x := x - 1;
  { Zdaj je 2^(k-1) <= x < 2^k.  Torej je najviji prigani
    bit v dvojikem zapisu tevila x ravno bit k-1.
    Naslednji stavki (ki predpostavljajo, da je tip integer
    dolg 32 bitov) razmaejo ta prigani bit e po vseh
    nijih mestih. }
  x := x or (x shr 1);
  x := x or (x shr 2);
  x := x or (x shr 4);
  x := x or (x shr 8);
  x := x or (x shr 16);
  { Zdaj so v x prigani vsi biti od 0 do k-1, torej je
    x = 2^k - 1.  Torej je x + 1 = 2^k, to pa je ravno
    tista vrednost, ki jo iemo. }
  Zaokrozi5 := x + 1;
end; {Zaokrozi5}

function Zaokrozi5a(x: integer): integer;
var r: integer;
begin
  x := x - 1; r := 1;
  while x and (x + 1) <> 0 do begin
    x := x or (x shr r);
    r := r shl 1; 
{$IFDEF STEJ_ITERACIJE}
    Inc(ni);
{$ENDIF}
  end; {while}
  Zaokrozi5a := x + 1;
end; {Zaokrozi5a}

{ Predpostavimo, da je tip integer dolg 32 bitov.
  Naa funkcija ima torej le 32 monih vrednosti (0, 1, 2, 4,
  8, 16, ..., 2^31) in vsako od njih mora vrniti pri tevilih
  z nekega intervala: 0 pri nepozitivnih tevilih, 1 pri 1,
  2 pri 2, 4 pri 3..4, 8 pri 5..8, 16 pri 9..16, ...,
  2^31 pri tevilih 1073741825..2147483648.  Zdaj lahko kar
  z bisekcijo ugotovimo, na katerem od teh intervalov lei na x. }
function Zaokrozi6(x: integer): integer;
begin
  if x <= 65536 then
    if x <= 256 then
      if x <= 16 then
        if x <= 4 then 
          if x <= 2 then  
            if x <= 0 then Zaokrozi6 := 0
            else Zaokrozi6 := x
          else Zaokrozi6 := 4
        else if x <= 8 then Zaokrozi6 := 8
        else Zaokrozi6 := 16
      else if x <= 64 then
        if x <= 32 then Zaokrozi6 := 32
        else Zaokrozi6 := 64
      else if x <= 128 then Zaokrozi6 := 128
      else Zaokrozi6 := 256
    else if x <= 4096 then 
      if x <= 1024 then
        if x <= 512 then Zaokrozi6 := 512
        else Zaokrozi6 := 1024
      else if x <= 2048 then Zaokrozi6 := 2048
      else Zaokrozi6 := 4096
    else if x <= 16384 then
      if x <= 8192 then Zaokrozi6 := 8192
      else Zaokrozi6 := 16384
    else if x <= 32768 then Zaokrozi6 := 32768
    else Zaokrozi6 := 65536
  else if x <= 16777216 then
    if x <= 1048576 then
      if x <= 262144 then
        if x <= 131072 then Zaokrozi6 := 131072
        else Zaokrozi6 := 262144
      else if x <= 524288 then Zaokrozi6 := 524288
      else Zaokrozi6 := 1048576
    else if x <= 4194304 then
      if x <= 2097152 then Zaokrozi6 := 2097152
      else Zaokrozi6 := 4194304
    else if x <= 8388608 then Zaokrozi6 := 8388608
    else Zaokrozi6 := 16777216
  else if x <= 268435456 then
    if x <= 67108864 then
      if x <= 33554432 then Zaokrozi6 := 33554432
      else Zaokrozi6 := 67108864
    else if x <= 134217728 then Zaokrozi6 := 134217728
    else Zaokrozi6 := 268435456
  else if x <= 1073741824 then
    if x <= 536870912 then Zaokrozi6 := 536870912 
    else Zaokrozi6 := 1073741824 
  else Zaokrozi6 := $80000000;
end; {Zaokrozi6}

function Zaokrozi6a(x: integer): integer;
var k1, k2, k: integer;
begin
  if x <= 0 then begin Zaokrozi6a := 0; exit end;
  k1 := 0; k2 := 32;
  while k2 - k1 > 1 do begin
    { Na tem mestu velja: 2^k1 <= x < 2^k2. }
    k := (k1 + k2) div 2;
    if x < 1 shl k then k2 := k else k1 := k;
  end; {while}
  { Na tem mestu velja: 2^k1 <= x < 2^(k1 + 1). }
  k := 1 shl k1;
  if x > k then k := k * 2;
  Zaokrozi6a := k;
end; {Zaokrozi6a}

function Zaokrozi7(x: integer): integer;
var y: integer;
begin
  if x <= 1 then y := 1
  else begin
    y := 1 shl Trunc(Ln(x) / Ln(2.0));
    if y < x then y := y shl 1;
  end; {else}
  Zaokrozi7 := y;
end; {Zaokrozi7}

function Zaokrozi7a(x: integer): integer;
var y: integer;
begin
  if x <= 1 then y := 1
  else begin
    y := 1 shl Trunc(Ln(x) * 1.4426950408889634);
    if y < x then y := y shl 1;
  end; {else}
  Zaokrozi7a := y;
end; {Zaokrozi7a}

function Zaokrozi8(x: integer): integer;
var r, y: integer;
begin
  if x <= 0 then begin Zaokrozi8 := 0; exit; end;
  r := 1 shl 29; y := r;
  while r >= x do begin
    r := r shr 1; 
{$IFDEF STEJ_ITERACIJE}
    Inc(ni);
{$ENDIF}
  end; {while}
  if x = 1 then Zaokrozi8 := 1
  else Zaokrozi8 := r * 2;
end; {Zaokrozi8}

function Zaokrozi8a(x: integer): integer;
begin
  if x > $40000000 then Zaokrozi8a := $80000000
  else if x > $20000000 then Zaokrozi8a := $40000000
  else if x > $10000000 then Zaokrozi8a := $20000000
  else if x > $08000000 then Zaokrozi8a := $10000000
  else if x > $04000000 then Zaokrozi8a := $08000000
  else if x > $02000000 then Zaokrozi8a := $04000000
  else if x > $01000000 then Zaokrozi8a := $02000000
  else if x > $00800000 then Zaokrozi8a := $01000000
  else if x > $00400000 then Zaokrozi8a := $00800000
  else if x > $00200000 then Zaokrozi8a := $00400000
  else if x > $00100000 then Zaokrozi8a := $00200000
  else if x > $00080000 then Zaokrozi8a := $00100000
  else if x > $00040000 then Zaokrozi8a := $00080000
  else if x > $00020000 then Zaokrozi8a := $00040000
  else if x > $00010000 then Zaokrozi8a := $00020000
  else if x > $00008000 then Zaokrozi8a := $00010000
  else if x > $00004000 then Zaokrozi8a := $00008000
  else if x > $00002000 then Zaokrozi8a := $00004000
  else if x > $00001000 then Zaokrozi8a := $00002000
  else if x > $00000800 then Zaokrozi8a := $00001000
  else if x > $00000400 then Zaokrozi8a := $00000800
  else if x > $00000200 then Zaokrozi8a := $00000400
  else if x > $00000100 then Zaokrozi8a := $00000200
  else if x > $00000080 then Zaokrozi8a := $00000100
  else if x > $00000040 then Zaokrozi8a := $00000080
  else if x > $00000020 then Zaokrozi8a := $00000040
  else if x > $00000010 then Zaokrozi8a := $00000020
  else if x > $00000008 then Zaokrozi8a := $00000010
  else if x > $00000004 then Zaokrozi8a := $00000008
  else if x > $00000002 then Zaokrozi8a := $00000004
  else if x > $00000001 then Zaokrozi8a := $00000002
  else if x > $00000000 then Zaokrozi8a := $00000001
  else Zaokrozi8a := 0;
end; {Zaokrozi8a}

function FtToLi(t: FILETIME): int64;
var li: LARGE_INTEGER;
begin
  li.LowPart := t.dwLowDateTime;
  li.HighPart := t.dwHighDateTime;
  FtToLi := li.QuadPart;
end;

function Cas: int64;
var ct, et, kt, ut: FILETIME;
begin
  GetProcessTimes(GetCurrentProcess, ct, et, kt, ut);
  Cas := FtToLi(kt) + FtToLi(ut);
end; {Cas}

type FunkcijaT = function(x: integer): integer;
var f: FunkcijaT; a, b, x, y, M, mm: integer; t: int64; st, st2: double;
begin {Zaokrozanje}

  { Katero funkcijo naj uporabimo? }
  f := @Zaokrozi1;
  if ParamCount >= 1 then
    if ParamStr(1) = '1a' then f := @Zaokrozi1a
    else if ParamStr(1) = '1b' then f := @Zaokrozi1b
    else if ParamStr(1) = '2' then f := @Zaokrozi2
    else if ParamStr(1) = '3' then f := @Zaokrozi3
    else if ParamStr(1) = '4' then f := @Zaokrozi4
    else if ParamStr(1) = '5' then f := @Zaokrozi5
    else if ParamStr(1) = '5a' then f := @Zaokrozi5a
    else if ParamStr(1) = '6' then f := @Zaokrozi6
    else if ParamStr(1) = '6a' then f := @Zaokrozi6a
    else if ParamStr(1) = '7' then f := @Zaokrozi7
    else if ParamStr(1) = '7a' then f := @Zaokrozi7a
    else if ParamStr(1) = '8' then f := @Zaokrozi8
    else if ParamStr(1) = '8a' then f := @Zaokrozi8a;
    
  { e imamo e dva parametra, a in b, kliimo f
    za vse x od a do b.  To je koristno za primerjavo
    asa izvajanja gornjih treh funkcij. }
  if ParamCount >= 3 then begin
    Val(ParamStr(2), a, x);
    Val(ParamStr(3), b, x);
    if ParamCount >= 4 then Val(ParamStr(4), M, x) else M := 1;
    st := 0; st2 := 0;
    for mm := 1 to M do begin
      t := Cas;
      y := 0;
      ni := 0;
      for x := a to b do 
        y := (y + f(x)) and $fffffff;
        {y := (y + (int64(x) * int64(f(x))) mod 99999989) and $fffffff;}
      t := Cas - t; st := st + double(t); st2 := st2 + double(t) * double(t);
    end; {for mm}
    WriteLn('Kontrolna vsota za ', a, '..', b, ': ', y);
    WriteLn('St. iteracij: ', ni);
    st := st / m; Write('Cas izvajanja (po ', M, ' poskusih): ', (st / 10000000.0):0:6, ' +/- ');
    st2 := st2 / m - st * st; if st2 <= 0 then st2 := 0 else st2 := Sqrt(st2);
    WriteLn((st2 / 10000000.0):0:6, ' s');
  end
  
  { Drugae pa le izpiimo f(x) za dani x. }
  else begin
    if ParamCount >= 2 then Val(ParamStr(2), x, a)
    else begin Write('Vnesi x: '); ReadLn(x); end;
    WriteLn('Zaokrozi(', x, ') = ', f(x));
  end; {if}
  
end. {Zaokrozanje}

{ Meritve (klicali smo Zaokrozi*(x) za vse x od 1 do N):
                     as izvajanja (samo as CPU)
  funkcija    N = 2^20      2^24      2^27     2^30
  1             155.4 ms    3.081 s   28.53 s  252.04 s
  1a            163.1       3.182     28.12    250.01
  1b            121.5       2.278     20.37    179.09
  2              73.7       1.346     11.73    101.81
  3              54.0       0.956      8.20     69.75
  4              62.8       1.003      8.03     64.17
  5              40.9       0.654      5.25     41.89
  5a             37.1       0.607      4.95     40.09
  6              15.6       0.235      1.84     14.59
  6a             81.5       1.307     10.10     78.90
  7             229.3       3.673     29.37    268.37
  7a            214.0       3.432     27.43    257.10
  }
