program Kamera; { RTK 2000.3.1 }
{ Ta program upoteva navodila o obliki
  vhodnih in izhodnih podatkov, kot jih
  doloa naloga #588 na acm.uva.es.
  http://acm.uva.es/p/v5/588.html }
  
const MaxN = 100;
var n: integer;
    px, py: array[0..MaxN] of integer; 

{ Reitev, ki primerja vsak par vzporednih stranic.
  asovna zahtevnost je zato O(n^2). }
function LahkoPostavimoKamero1: boolean; 
var i, j: integer; 
begin
  LahkoPostavimoKamero1 := false; 
  for i := 0 to n - 1 do begin 
    { Stranici i--(i+1) so vzporedne stranice (i+2)--(i+3), 
      (i+4)--(i+5), itd., pa tudi (i-2)--(i-1) ipd., skratka
      vse j--(j+1), za katere je i mod 2 = j mod 2. }
    j := i mod 2; while j < n do begin 
      if px[i] = px[i + 1] then begin { navpino } 
        if (px[i] < px[j]) and (py[i] > py[i + 1]) and (py[j] < py[j + 1]) then exit; 
      end else                        { vodoravno } 
        if (py[i] > py[j]) and (px[i] > px[i + 1]) and (px[j] < px[j + 1]) then exit; 
      j := j + 2; 
    end; {while}
  end; {for}
  LahkoPostavimoKamero1 := true; 
end; 

{ Reitev, ki za vsako stranico definira po eno
  omejitev in preveri, e so skladne.  asovna
  zahtevnost je zato le O(n). }
function LahkoPostavimoKamero2: boolean; 
var i, xMin, yMin, xMax, yMax: integer; 
begin
  { Na zaetku vzemimo, da je sprejemljiv vsak poloaj kamere 
    v ortanem pravokotniku (bounding box) cele sobe.}
  xMin := px[1]; xMax := px[1]; yMin := py[1]; yMax := py[1]; 
  for i := 2 to n do begin
    if px[i] < xMin then xMin := px[i]; 
    if px[i] > xMax then xMax := px[i]; 
    if py[i] < yMin then yMin := py[i]; 
    if py[i] > yMax then yMax := py[i]; 
  end; {for}
  { Upotevajmo omejitve, ki jih nalagajo posamezne stene. } 
  for i := 1 to n do               { Oglejmo si steno od i-1 do i. } 
    if px[i] = px[i - 1] then begin    { navpina } 
      if (py[i] > py[i - 1]) and (px[i] > xMin) then xMin := px[i]; 
      if (py[i] < py[i - 1]) and (px[i] < xMax) then xMax := px[i]; 
    end else begin     { vodoravna } 
      if (px[i] < px[i - 1]) and (py[i] > yMin) then yMin := py[i]; 
      if (px[i] > px[i - 1]) and (py[i] < yMax) then yMax := py[i]; 
    end; {if} 
  { Sprejemljivi poloaji kamere so vsi, ki ustrezajo pogojema 
    xMin < xKamere < xMax in yMin < yKamere < yMax. 
    Torej mora biti xMin < xMax in yMin < yMax. }
  LahkoPostavimoKamero2 := (xMin < xMax) and (yMin < yMax);
  { Ocenjevalni strenik acm.uva.es pa reitev    A---B    
    zavraa, e v pogoj ne vkljuimo tudi         |   |    
    enakosti.  To je sicer precej nenavadno, saj  |   C---D
    pomeni, da je mogo video nadzor tudi v sobi  |       |
    s tlorisom, ki je prikazan na desni, pa      H---G   |
    tako, da je kamera nekje na daljici CG.           |   |
                                                      F---E 
    Gornji podprogram LahkoPostavimoKamero1 pa tako ali tako
    e zdaj deluje na ta nain, saj bi npr. pri stranicah 
    BC in FG preveril le, e ni sluajno BC strogo levo od
    FG; e sta na enaki X-koordinati, ga to ne moti.  e bi
    hoteli to odpraviti, bi morali v (px[i] < px[j]) in
    (py[i] > py[j]) v tistem podprogramu dotati e enaaj. }
  LahkoPostavimoKamero2 := (xMin <= xMax) and (yMin <= yMax); 
end; {LahkoPostavimoKamero2}

var i, StPrimera: integer;
begin {Kamera}
  StPrimera := 0;
  while true do begin
    { Preberimo naslednji testni primer. }
    ReadLn(n); 
    if n = 0 then break;
    for i := 1 to n do ReadLn(px[i], py[i]);
    px[0] := px[n]; py[0] := py[n];
    { Izpiimo odgovor. }
    StPrimera := StPrimera + 1;
    WriteLn('Floor #', StPrimera);
    Write('Surveillance is ');
    if not LahkoPostavimoKamero1 then Write('im');
    WriteLn('possible.'); WriteLn;
  end; {while}
end. {Kamera}

{ Na streniku na acm.uva.es se program izvaja enako dolgo
  tako pri LahkoPostavimoKamero1 kot pri LahkoPostavimoKamero2.
  Oitno N <= 100 ogli v tem primeru e ni dovolj, da bi 
  se razlika med kvadratno in linearno zahtevnostjo poznala 
  tudi v praksi. }

