program PosiljanjeSporocil; { RTK 2003.2.4 }
{ Pri tej nalogi je treba pripraviti podprogram, 
  ki bi ga hkrati izvajalo ve raunalnikov, med
  seboj pa bi si poiljali sporoila.  V tem programu
  moramo takno okolje seveda simulirati. 
  Podprogram ObPrejemuSporocila dobi tevilko raunalnika,
  na katerem naj si misli, da se izvaja, kar kot parameter. 
  Glavni del programa simulira dogajanje v mrei in
  izpisuje podatke o tem, kdaj kdo prevzame kakno sporoilo. }

const MaxStRacunalnikov = 100;
type 
  NaslovT = integer;
  SporociloT = record 
    PrvoSporocilo: boolean; 
    { Naslov prvega raunalnika (tistega, ki je prvi dobil  
      sporoilo od uporabnika).  Tega potrebujemo, da lahko  
      vsak raunalnik sestavi enak vrstni red naslovov in to 
      takega, v katerem je prvi raunalnik na zaetku tabele. }
    NaslovPrvega: NaslovT; 
  end; {SporociloT}
  
  procedure PosljiSporocilo(S: SporociloT; 
                            Posiljatelj, Prejemnik: NaslovT); forward;
  
  type NasloviT = array[1..MaxStRacunalnikov] of NaslovT;
  procedure NasloviVsehRacunalnikov(var Naslovi: NasloviT); forward;

  var StRacunalnikov: integer;

{-----------------------------------------------------------------------------}
{ Tu je podprogram ObPrejemuSporocila, kot naj bi ga opisali tekmovalci.      }
{-----------------------------------------------------------------------------}

procedure ObPrejemuSporocila(NasNaslov: NaslovT; 
                             S: SporociloT; Posiljatelj: NaslovT); 

  procedure PrerazporediNaslove(var Naslovi: NasloviT; NaslovPrvega: NaslovT); 
  var Naslovi2: NasloviT; i, j: integer; 
  begin
    { Naslove, ki so manji kot NaslovPrvega, odloimo v pomono tabelo. } 
    i := 0; while Naslovi[i + 1] <> NaslovPrvega do 
      begin Inc(i); Naslovi2[i] := Naslovi[i]; end;
    { Naslove, ki so veji ali enaki NaslovPrvega, premaknemo na zaetek tabele. } 
    for j := i + 1 to StRacunalnikov do Naslovi[j - i] := Naslovi[j]; 
    { Na konec tabele zapiemo naslove, manje od NaslovPrvega. } 
    for j := 1 to i do Naslovi[StRacunalnikov - i + j] := Naslovi2[j]; 
  end; {PrerazporediNaslove} 

  function IndeksNaslovaVTabeli(Naslov: NaslovT; var Naslovi: NasloviT): integer; 
  var i: integer; 
  begin
    i := 1; while Naslovi[i] <> Naslov do Inc(i); 
    IndeksNaslovaVTabeli := i; 
  end; {IndeksNaslovaVTabeli} 

var 
  Naslovi: NasloviT; 
  Indeks, k: integer; 
begin
  { e je tole uporabnikovo sporoilo, vemo, da je  
    na raunalnik prvi, ki je bil obveen. }
  if S.PrvoSporocilo then begin
    S.PrvoSporocilo := false; 
    S.NaslovPrvega := NasNaslov; 
  end; {if}
  { Spomnimo se, da nam podprogram NasloviVsehRacunalnikov 
    vrne naslove, urejene naraajoe.  Torej bo vsak 
    raunalnik po klicu te funkcije videl enak vrstni red. }
  NasloviVsehRacunalnikov(Naslovi); 
  { PrerazporediNaslove premakne naslove, manje od 
    S.NaslovPrvega, na konec tabele, drugae pa njihovega 
    medsebojnega vrstnega reda ni ne spreminja. }
  PrerazporediNaslove(Naslovi, S.NaslovPrvega); 
  { IndeksNaslovaVTabeli vrne indeks (1..StRacunalnikov) 
    danega naslova v dani tabeli. } 
  Indeks := IndeksNaslovaVTabeli(NasNaslov, Naslovi) - 1; 
  { Izraunajmo k, torej v kateri sekundi smo mi prejeli tole sporoilo. } 
  k := 0; while not (Indeks < 1 shl k) do Inc(k); 
  { V naslednji, torej (k + 1)-vi sekundi, bomo morali 
    obvestiti raunalnik Indeks + 1 shl k. }
  while Indeks + 1 shl k < StRacunalnikov do begin 
    Inc(k); { Zdaj smo v novi sekundi. } 
    { V k-ti sekundi obvestimo raunalnik Indeks + 1 shl (k - 1).  
      Vendar pa ne pozabimo, da mi tejemo indekse od 0 naprej, 
      v tabeli Naslovi pa so od 1 naprej. }
    PosljiSporocilo(S, NasNaslov, Naslovi[Indeks + 1 shl (k - 1) + 1]); 
  end; {while}
end; {ObPrejemuSporocila} 

{-----------------------------------------------------------------------------}
{ Implementacija podprogramov za komunikacijo z drugimi raunalniki           }
{-----------------------------------------------------------------------------}

type
  ClenP = ^ClenT;
  ClenT = record
    Posiljatelj: NaslovT;
    CasPrejema: integer;
    S: SporociloT;
    Nasl: ClenP;
  end; {ClenT}
  
  VrstaT = record
    Glava, Rep: ClenP;
  end; {VrstaT}

var 
  { Za vsak raunalnik imamo vrsto sporoil, ki jih je
    dobil, prevzel oz. obdelal pa e ne. }
  Vrste: array[1..MaxStRacunalnikov] of VrstaT;
  { Casi[i] pove, v katerem asovnem koraku bi se zdajle
    e nahajal raunalnik i, e bi poiljanje vsakega 
    sporoila trajalo cel asovni korak.  Glej komentar
    spodaj pri podprogramu PosljiSporocilo. }
  Casi: array[1..MaxStRacunalnikov] of integer;

{ Indeksi raunalnikov v tabelah Vrste in Casi gredo od 1 do N.
  Da pa naslovi, kot jih vidi npr. podprogram ObPrejemuSporocila,
  ne bi bili prelepi, bomo za preraunavanje med indeksi in
  naslovi uporabili spodnji dve formuli. }
function NaslovVIndeks(Naslov: NaslovT): integer;
  begin NaslovVIndeks := ((Naslov xor $5555) - 5) div 3; end;
function IndeksVNaslov(Indeks: integer): NaslovT;
  begin IndeksVNaslov := (Indeks * 3 + 5) xor $5555; end;

{ Naloga pravi, da klic podprograma PosljiSporocilo popolnoma
  zaposli raunalnik-poiljatelja za en asovni korak (eno sekundo).  
  Tukaj si esa podobnega seveda ne moremo privoiti, ker moramo v
  okviru istega asovnega koraka simulirati tudi delovanje drugih
  raunalnikov, ki v tem koraku prejemajo kakno sporoilo. 
  Zato bomo dovolili, da se ObPrejemuSporocila izvede takoj v celoti,
  mi pa si bomo v spremenljivki Casi[i] zapomnili, v kateri sekundi
  bi se e nahajal raunalnik i, e bi poiljanje vsakega sporoila
  res trajalo eno sekundo.  Na podlagi vrednosti Casi[i] lahko tudi
  doloimo, v kateri sekundi naj raunalnik-poiljatelj 
  opazi oz. prevzame to sporoilo (ta as zapiemo v Clen^.CasPrejema,
  kar bo upotevala glavna zanka v glavnem delu programa). }
procedure PosljiSporocilo(S: SporociloT; Posiljatelj, Prejemnik: NaslovT); 
var Clen: ClenP; i, j, CasPrejema: integer;
begin
  { Poveajmo poiljateljev as.  Poseben primer je 
    Posiljatelj = -1 (zaetno sporoilo, ki prihaja 
    od uporabnika). }
  if Posiljatelj >= 0 then begin
    i := NaslovVIndeks(Posiljatelj); 
    Inc(Casi[i]); CasPrejema := Casi[i];
  end else CasPrejema := 1;
  { Dodajmo novo sporoilo v prejemnikovo vrsto. }
  j := NaslovVIndeks(Prejemnik);
  with Vrste[j] do begin
    New(Clen); Clen^.S := S; 
    Clen^.Posiljatelj := Posiljatelj;
    Clen^.CasPrejema := CasPrejema;
    Clen^.Nasl := nil;
    if Rep = nil then Glava := Clen 
    else Rep^.Nasl := Clen;
    Rep := Clen;
  end; {with}
end;

procedure NasloviVsehRacunalnikov(var Naslovi: NasloviT); 
var i, j: integer; Naslov: NaslovT;
begin
  { Naslove pripravimo s funkcijo IndeksVNaslovi,
    uredimo pa jih z vstavljanjem (insertion sort). }
  for i := 1 to StRacunalnikov do begin
    Naslov := IndeksVNaslov(i);
    j := i - 1;
    while j > 0 do
      if Naslovi[j] <= Naslov then break 
      else begin Naslovi[j + 1] := Naslovi[j]; Dec(j); end;
    Naslovi[j + 1] := Naslov;
  end; {for}
end; {NasloviVsehRacunalnikov}

{-----------------------------------------------------------------------------}
{ Glavni del programa (simulacija dogajanja v omreju)                        }
{-----------------------------------------------------------------------------}

var i, Cas: integer; S: SporociloT; 
    Prej, Clen, Nasl: ClenP; Koncano: boolean; 
    Prvi: NaslovT; Naslovi: NasloviT;
begin {PosiljanjeSporocil}

  { Doloimo tevilo raunalnikov. }
  if ParamCount >= 1 then begin
    Val(ParamStr(1), StRacunalnikov, i);
    WriteLn('tevilo raunalnikov: ', StRacunalnikov);
  end else begin
    Write('tevilo raunalnikov: ');
    ReadLn(StRacunalnikov);
  end; {if}
  { Izpiimo vse naslove. }
  NasloviVsehRacunalnikov(Naslovi); Write('Naslovi:');
  for i := 1 to StRacunalnikov do Write(' ', Naslovi[i]);
  WriteLn;
  
  { Na zaetku so vse vrste sporoil prazne. }
  for i := 1 to MaxStRacunalnikov do with Vrste[i] do
    begin Glava := nil; Rep := nil; end;
    
  { Poljimo zaetno sporoilo enemu od raunalnikov. }
  Cas := 0; WriteLn('asovni korak ', Cas);
  S.PrvoSporocilo := true; S.NaslovPrvega := -1;
  Prvi := IndeksVNaslov(Random(StRacunalnikov) + 1);
  WriteLn('Uporabnik polje sporoilo raunalniku ', Prvi, '.');
  PosljiSporocilo(S, -1, Prvi);
  
  { Simulirajmo nadaljnje dogajanje. }
  repeat
    Inc(Cas); WriteLn('asovni korak ', Cas);
    Koncano := true;
    { Preglejmo vse raunalnike in obdelajmo sporoila, ki 
      jih je treba prevzeti v tem asovnem koraku. }
    for i := 1 to StRacunalnikov do begin
      Prej := nil; Clen := Vrste[i].Glava;
      if Clen <> nil then Koncano := false;
      { Preglejmo vsa sporoila v vrsti raunalnika i. }
      while Clen <> nil do begin
        Nasl := Clen^.Nasl;
        if Clen^.CasPrejema <> Cas then Prej := Clen
        else begin
          { as je, da raunalnik i prevzame tole sporoilo. }
          with Clen^.S do WriteLn(IndeksVNaslov(i), ' prevzame sporoilo (', 
            PrvoSporocilo, ', ', NaslovPrvega, '), ki ga je poslal ', Clen^.Posiljatelj, '.');
          Casi[i] := Cas;
          ObPrejemuSporocila(IndeksVNaslov(i), Clen^.S, Clen^.Posiljatelj);
          { Prevzeto sporoilo pobriimo iz vrste. }
          if Prej <> nil then Prej^.Nasl := Nasl 
          else Vrste[i].Glava := Nasl;
          Dispose(Clen);
        end; {if}
        Clen := Nasl; { pojdimo naprej po vrsti akajoih sporoil }
      end; {while}
    end; {for}
    { e so bile vse vrste prazne, so tudi ostale prazne
      in lahko konamo s simuliranjem. }
  until Koncano;
  
  WriteLn('Konec simulacije.');
end. {PosiljanjeSporocil}
