program TekoceStopnice; { RTK 2004.1.4 }
uses Crt;

const CasVoznje = 10;
type SenzorT = (Zgoraj, Spodaj);
     SmerT = (Gor, Dol, Ustavi);
procedure PozeniStopnice(NovaSmer: SmerT); forward;
function PotnikNaSenzorju(Senzor: SenzorT): boolean; forward;

{-----------------------------------------------------------------------------}
{ Podprogram, kakrnega naj bi ga napisali tekmovalci                         }
{-----------------------------------------------------------------------------}

var TrenutnaSmer, ZadnjaSmer: SmerT;
    KakoDolgoSe: integer;

procedure Inicializacija;
begin
  TrenutnaSmer := Ustavi;
  ZadnjaSmer := Ustavi;
  KakoDolgoSe := 0;
end; {Inicializacija}

procedure EnkratNaMilisekundo;
begin
  if TrenutnaSmer <> Ustavi then begin
  
    KakoDolgoSe := KakoDolgoSe - 1;
    
    if ((TrenutnaSmer = Gor) and PotnikNaSenzorju(Spodaj))
    or ((TrenutnaSmer = Dol) and PotnikNaSenzorju(Zgoraj)) 
    then KakoDolgoSe := CasVoznje * 1000;
    
    if KakoDolgoSe <= 0 then 
      begin PozeniStopnice(Ustavi); TrenutnaSmer := Ustavi end;
      
  end 
  else begin
  
    if PotnikNaSenzorju(Spodaj) then
      if PotnikNaSenzorju(Zgoraj) then
        if ZadnjaSmer = Gor then TrenutnaSmer := Dol
        else TrenutnaSmer := Gor
      else TrenutnaSmer := Gor
    else if PotnikNaSenzorju(Zgoraj) then
      TrenutnaSmer := Dol;
    
    if TrenutnaSmer <> Ustavi then begin
      KakoDolgoSe := CasVoznje * 1000;
      ZadnjaSmer := TrenutnaSmer;
      PozeniStopnice(TrenutnaSmer);
    end; {if}
    
  end; {if}
end; {EnkratNaMilisekundo}

{-----------------------------------------------------------------------------}
{ Simulacija tekoih stopnic in implementacija podprogramov, za katere        }
{ naloga pravi, naj predpostavimo, da so e podani                            }
{-----------------------------------------------------------------------------}

const 
  { CasVoznje mora biti v resnici dalji od pravega asa vonje, ker ga
    uporablja EnkratNaMilisekundo za doloanje tega, kako dolgo se bodo
    stopnice premikale.  e bi bil CasVoznje dejanski as vonje, bi se
    lahko zgodilo, da bi se mi ravno pripravili na izklop stopnic, potnik
    pa bi e stal ob senzorju na koncu stopnic, kjer bo zdaj izstopil,
    in mi bi e mislili, da senzor zaznava novega potnika (ki prihaja iz
    nasprotne smeri kot prejnji), in bi stopnice zato po nepotrebnem
    pognali v nasprotno smer (namesto da bi jih ustavili). }
  PraviCasVoznje = CasVoznje - 1;
  { SirinaPotnika doloa, na katerem intervalu koordinat bosta senzorja
    zaznavala potnike.  Gornji senzor ima koordinato 0, spodnji 1, 
    potnike pa zaznavata, e imajo koordinato za <= 0.5 SirinaPotnika
    stran od senzorja.  Spodnja vrednost torej pomeni, da senzor potnika
    vidi priblino eno desetinko sekunde. }
  SirinaPotnika = 1.0 / (10 * PraviCasVoznje);
type
  PotnikP = ^PotnikT;
  PotnikT = record
    Polozaj: real; { 0 = na zgornjem koncu stopnic (levo), 1 = na spodnjem (desno) }
    Znak: char;
    Nasl: PotnikP;
  end; {PotnikT}

var 
  Potniki: PotnikP;
  Smer: SmerT;
  PotnikSpodaj, PotnikZgoraj: boolean;
  Cas: integer;

procedure PozeniStopnice(NovaSmer: SmerT); 
begin
  Smer := NovaSmer;
end; {PozeniStopnice}

function PotnikNaSenzorju(Senzor: SenzorT): boolean; 
begin
  if Senzor = Zgoraj then PotnikNaSenzorju := PotnikZgoraj
  else PotnikNaSenzorju := PotnikSpodaj;
end; {PotnikNaSenzorju}

{ Prikae stanje stopnic.  Potniki so oznaeni s rkami A..Z;
  na levi je vrednost KakoDolgoSe, na desni pa stanje senzorjev
  (rki L in D, e levi oz. desni senzor zaznava kaknega potnika)
  in smer gibanja stopnic (<<< ali >>>). }
procedure PrikaziStanje;
var S, T: string; P: PotnikP; i: integer;
begin
  Str(KakoDolgoSe, T); while Length(T) < 5 do T := ' ' + T;
  S := '     __________________________________________________     ';
  P := Potniki;
  while P <> nil do begin
    i := Round(P^.Polozaj * 50) + 5;
    if (i >= 0) and (i < Length(S)) then S[i + 1] := P^.Znak;
    P := P^.Nasl;
  end; {while}
  S := T + S;
  if PotnikZgoraj then S := S + ' L' else S := S + '  ';
  if PotnikSpodaj then S := S + ' D' else S := S + '  ';
  if Smer = Gor then S := S + ' <<<' 
  else if Smer = Dol then S := S + ' >>>'
  else S := S + '    ';
  Write(S, Chr(13));
end; {PrikaziStanje}

procedure SimulirajEnoMilisekundo;
var P, Nasl, PrejP: PotnikP; Premik: real;
begin
  if Smer = Ustavi then Premik := 0.0
  else if Smer = Gor then Premik := -1.0 / (1000.0 * PraviCasVoznje)
  else if Smer = Dol then Premik := 1.0 / (1000.0 * PraviCasVoznje);
  P := Potniki; PrejP := nil;
  PotnikSpodaj := false; PotnikZgoraj := false;
  { Poglejmo, kako se zaradi gibanja stopnic premikajo potniki.
    e kak potnik pride do konca stopnic, ga bomo pobrisali.
    Ugotovili bomo tudi, e senzorja zaznavata kaknega potnika. }
  while P <> nil do begin
    P^.Polozaj := P^.Polozaj + Premik;
    
    if (P^.Polozaj < -0.5 * SirinaPotnika) 
    or (P^.Polozaj > 1 + 0.5 * SirinaPotnika) then begin
      { Ta potnik je priel do konca stopnic, zato ga pobriimo. }
      Nasl := P^.Nasl;
      if PrejP = nil then Potniki := Nasl else PrejP^.Nasl := Nasl;
      Dispose(P); P := Nasl; continue;
    end; {if}
    
    if P^.Polozaj <= 0.5 * SirinaPotnika then PotnikZgoraj := true;
    if P^.Polozaj >= 1.0 - 0.5 * SirinaPotnika then PotnikSpodaj := true;
    PrejP := P; P := P^.Nasl;
  end; {while}
  
  EnkratNaMilisekundo;
  
  if (Cas mod 5) = 0 then PrikaziStanje;
  Cas := Cas + 1;
  
end; {SimulirajEnoMilisekundo}

var 
  c, NaslPotnik: char;
  P: PotnikP;
  Koncaj: boolean;
begin {TekoceStopnice}

  Inicializacija;
  Potniki := nil;
  Smer := Ustavi;
  NaslPotnik := 'A';
  Cas := 0;
  WriteLn('Simulator tekoih stopnic');
  WriteLn('L, D = polji novega potnika z leve/desne');
  WriteLn('P = pavza, Q ali X = izhod');
  WriteLn;

  Koncaj := false;
  while not Koncaj do begin
    SimulirajEnoMilisekundo;
    while KeyPressed do begin
      c := UpCase(ReadKey);
      if c = Chr(0) then c := UpCase(ReadKey)
      else if (c = Chr(27)) or (c = 'Q') or (c = 'X') then 
        begin Koncaj := true; break end
      else if (c = 'L') or (c = 'D') then begin
        New(P);
        if c = 'L' then P^.Polozaj := -0.4 * SirinaPotnika
        else P^.Polozaj := 1.0 + 0.4 * SirinaPotnika;
        P^.Znak := NaslPotnik;
        P^.Nasl := Potniki; Potniki := P;
        if NaslPotnik = 'Z' then NaslPotnik := 'A'
        else NaslPotnik := Succ(NaslPotnik);
      end else if c = 'P' then begin
        PrikaziStanje;
        Write('(PAVZA)', Chr(13));
        c := ReadKey; if c = Chr(0) then c := ReadKey;
        PrikaziStanje;
      end; {if}
    end; {while KeyPressed}
  end; {while not Koncaj}
  WriteLn;

  { Poistimo morebitne preostale potnike. }
  while Potniki <> nil do begin
    P := Potniki^.Nasl;
    Dispose(Potniki); Potniki := P;
  end; {while}
end. {TekoceStopnice}
