program Izomorfizem; { RTK 2004.3.4 }
const MaxN = 1000;

type
  GrafT = record
    Stopnja: array [1..MaxN] of integer;
    Sosedje: array [1..MaxN, 1..3] of integer;
    { Ostala polja uporabljamo, ko grafu izberemo koren 
      in ga predelamo v drevo. }
    Koren: integer;
    Otrok: array [1..MaxN, 1..2] of integer;
    StOtrok: array [1..MaxN] of integer;
  end; {GrafT}

procedure PreberiGraf(var T: text; N: integer; var G: GrafT);
var i, u, v: integer;
begin
  for i := 1 to N do G.Stopnja[i] := 0;
  for i := 1 to N - 1 do begin
    ReadLn(T, u, v); {u := u + 1; v := v + 1;}
    Assert(G.Stopnja[u] < 3); Assert(G.Stopnja[v] < 3);
    G.Stopnja[u] := G.Stopnja[u] + 1; G.Sosedje[u, G.Stopnja[u]] := v;
    G.Stopnja[v] := G.Stopnja[v] + 1; G.Sosedje[v, G.Stopnja[v]] := u;
  end; {for i}
  { Preverimo, e je z grafom vse v redu. }
  for i := 1 to N do for u := 1 to G.Stopnja[i] - 1 do for v := u + 1 to G.Stopnja[i] do
    Assert(G.Sosedje[i, u] <> G.Sosedje[i, v]);
end; {PreberiGraf}

{ Ko grafu izberemo koren, lahko iz njega naredimo drevo in za vsako
  vozlie pogledamo, kdo so njegovi otroci oz. poddrevesa. }
procedure PostaviKoren(var G: GrafT; R: integer);

  procedure Rekurzija(u, Oce: integer);
  var i, v: integer;
  begin
    G.StOtrok[u] := 0;
    for i := 1 to G.Stopnja[u] do begin
      v := G.Sosedje[u, i]; if v = Oce then continue;
      Assert(G.StOtrok[u] < 2);
      G.StOtrok[u] := G.StOtrok[u] + 1; 
      G.Otrok[u, G.StOtrok[u]] := v;
      Rekurzija(v, u);
    end; {for i}
  end; {Rekurzija}

begin {PostaviKoren}
  {WriteLn('PostaviKoren(', R, ')');}
  Assert(G.Stopnja[R] < 3);
  G.Koren := R;
  Rekurzija(R, -1);
end; {PostaviKoren}

var Preslikava: array [1..MaxN] of integer;

function StaDrevesiIzomorfni(N: integer; var G1, G2: GrafT): boolean;

  { Ta podprogram preveri, e sta dve poddrevesi (s korenom 
    pri u in v) izomorfni. }
  function Preveri(u, v: integer): boolean;
  var Sta: boolean;
  begin
      if G1.StOtrok[u] <> G2.StOtrok[v] then Sta := false
      else begin
        if G1.StOtrok[u] = 0 then Sta := true
        else if G1.StOtrok[u] = 1 then 
          Sta := Preveri(G1.Otrok[u, 1], G2.Otrok[v, 1])
        else begin
          Assert(G1.StOtrok[u] = 2);
          Sta := (Preveri(G1.Otrok[u, 1], G2.Otrok[v, 1]) and
                  Preveri(G1.Otrok[u, 2], G2.Otrok[v, 2])) or
                 (Preveri(G1.Otrok[u, 1], G2.Otrok[v, 2]) and
                  Preveri(G1.Otrok[u, 2], G2.Otrok[v, 1]));
        end; {if}
        if Sta then Preslikava[u] := v;
      end; {if}
    {WriteLn('Preveri(', u, ', ', v, ') = ', Sta);}
    Preveri := Sta;
  end; {Preveri}
  
begin {StaDrevesiIzomorfni}
  StaDrevesiIzomorfni := Preveri(G1.Koren, G2.Koren);
end; {StaDrevesiIzomorfni}

var
  T: text;
  u: integer; N: integer; G1, G2: GrafT; StaIzo: boolean;
begin {Izomorfizem}

  { Preberimo oba grafa. }
  if ParamCount >= 1 then Assign(T, ParamStr(1)) else Assign(T, 'izomorf.in');
  Reset(T); ReadLn(T, N);
  Assert(1 <= N); Assert(N <= MaxN);
  PreberiGraf(T, N, G1); PreberiGraf(T, N, G2);
  Close(T);
  
  { Izberimo koren prvega grafa. }
  u := 1; 
  while u <= N do if G1.Stopnja[u] < 3 then break else u := u + 1;
  Assert(u <= N);
  PostaviKoren(G1, u);
  
  { Preizkusimo mone korene drugega grafa. }
  u := 1; StaIzo := false; 
  while (u <= N) and not StaIzo do begin
    if G2.Stopnja[u] = G1.Stopnja[G1.Koren] then begin
      PostaviKoren(G2, u);
      StaIzo := StaDrevesiIzomorfni(N, G1, G2);
    end; {if}
    u := u + 1;
  end; {while}
    
  { Izpiimo rezultat. }
  if ParamCount >= 2 then Assign(T, ParamStr(2)) else Assign(T, 'izomorf.out');
  Rewrite(T); 
  if StaIzo then for u := 1 to N do WriteLn(T, u, ' ', Preslikava[u]);
  Close(T);

end. {Izomorfizem}
