program TrikotnikiIzDaljic;
const 
  MaxDaljic = 18000; MaxTock = 2 * MaxDaljic;
  Eps = 1e-6;
var
  Xi, Yi: array [1..MaxTock] of Extended;
  M: integer; { tevilo tok (kraji daljic) }

  { Koordinati toke i sta (Xi[i], Yi[i]).  Spodnji podprogram za dani koordinati
    poie indeks te toke; e take toke e nimamo v tabelah Xi in Yi, jo doda. }
  function StTocke(X, Y: Extended): integer;
  var i: integer;
  begin
    for i := 1 to M do 
      if (Abs(X - Xi[i]) < Eps) and (Abs(Y - Yi[i]) < Eps)
      then begin StTocke := i; exit end;
    M := M + 1; Xi[M] := X; Yi[M] := Y;  StTocke := M;
  end; {StTocke}

  { Spodnja funkcija ugotovi orientacijo trikotnika ABC (pozitivna je, e so
    oglia navedena v smeri, nasprotni smeri urinega kazalca). }
  function Ccw(A, B, C: integer): integer; { ccw = counterclockwise }
  var ABx, ABy, ACx, ACy, Z: Extended;
  begin
    ABx := Xi[B] - Xi[A]; ABy := Yi[B] - Yi[A]; { AB = vektor od A do B }
    ACx := Xi[C] - Xi[A]; ACy := Yi[C] - Yi[A]; { AC = vektor od A do C }
    Z := ABx * ACy - ACx * ABy; { vektorski produkt AB in AC je (0, 0, Z) }
    if Z > Eps then Ccw := 1 { pozitivna orientacija }
    else if Z < Eps then Ccw := -1 { negativna orientacija }
    else Ccw := 0; { toke so kolinearne, trikotnik je izrojen }
  end; {Ccw}

  function LeziV(T, A, B, C: integer): boolean; { Ali lei T v trikotniku ABC? }
  var i: integer;
  begin
    if Ccw(A, B, C) < 0 then begin i := B; B := C; C := i end;
    LeziV := (Ccw(A, B, T) >= 0) and (Ccw(B, C, T) >= 0) and (Ccw(C, A, T) >= 0);
  end; {LeziV}
  
{ Tabela z vsemi najdenimi trikotniki.  Vsak trikotnik ima tri stranice, vsaka
  daljica pa je lahko stranica najve dveh trikotnikov, torej je tevilo
  trikotnikov <= 2/3 tevila daljic in spodnja tabela bo gotovo dovolj velika. }
var nTri: integer; Tri: array [1..MaxDaljic, 1..3] of integer;
  
  procedure ShraniTrikotnik(A, B, C: integer);
  var i: integer;
  begin
    if C = 0 then exit;
    { Vsak trikotnik bomo nali trikrat: enega od ABC in BAC;
      enega od BCA in BAC; in enega od CAB in ACB. }
    if (C > B) or (C > A) then exit;
    { Pri izpisu zahtevamo pozitivno orientacijo. 
      e je trenutno orientiran negativno, zamenjajmo dve oglii. }
    if Ccw(A, B, C) < 0 then begin i := B; B := C; C := i end;
    { Zapomnimo si novi trikotnik. }
    nTri := nTri + 1; Tri[nTri, 1] := A; Tri[nTri, 2] := B; Tri[nTri, 3] := C;
  end; {ShraniTrikotnik}

var
  { Toka i ima StSosed[i] sosed, namre toke Sosede[PrvaSoseda[i] + j] 
    za j = 0, 1, ..., StSosed[i] - 1. }
  StSosed, PrvaSoseda: array [1..MaxTock] of integer;
  Sosede: array [1..2 * MaxDaljic] of integer;
  Krajisca: array [1..MaxDaljic, 1..2] of integer; { krajii vsake daljice }
  N: integer; { t. daljic }
  i, j, u, v, iu, iv, w, ww, Vrh1, Vrh2: integer;
  X1, Y1, X2, Y2: Extended;
  T: text;
begin {TrikotnikiIzDaljic}

  { Preberimo podatke o daljicah, izraunajmo stopnje. }
  if ParamCount >= 1 then Assign(T, ParamStr(1)) else Assign(T, 'tri.in');
  Reset(T);
  ReadLn(T, N); M := 0;
  for u := 1 to 2 * N do StSosed[u] := 0;
  for i := 1 to N do begin
    ReadLn(T, X1, Y1, X2, Y2); 
    u := StTocke(X1, Y1); v := StTocke(X2, Y2);
    Krajisca[i, 1] := u; Krajisca[i, 2] := v;
    StSosed[u] := StSosed[u] + 1;
    StSosed[v] := StSosed[v] + 1;
  end; {for i}
  Close(T);

  { Pripravimo sezname sosed. }
  PrvaSoseda[1] := 1;
  for u := 1 to M - 1 do PrvaSoseda[u + 1] := PrvaSoseda[u] + StSosed[u];
  for u := 1 to M do StSosed[u] := 0;
  for i := 1 to N do begin
    u := Krajisca[i, 1]; v := Krajisca[i, 2];
    Sosede[PrvaSoseda[u] + StSosed[u]] := v; StSosed[u] := StSosed[u] + 1;
    Sosede[PrvaSoseda[v] + StSosed[v]] := u; StSosed[v] := StSosed[v] + 1;
  end; {for i}
  
  { Uredimo vsak seznam sosed. }
  for u := 1 to M do begin
    for i := 1 to StSosed[u] - 1 do begin
      j := i - 1; v := Sosede[PrvaSoseda[u] + i];
      while j >= 0 do begin
        if Sosede[PrvaSoseda[u] + j] <= v then break;
        Sosede[PrvaSoseda[u] + j + 1] := Sosede[PrvaSoseda[u] + j]; j := j - 1;
      end; {while}
      Sosede[PrvaSoseda[u] + j + 1] := v;
    end; {while}
  end; {for i}
  
  nTri := 0;
  for i := 1 to N do begin
    u := Krajisca[i, 1]; v := Krajisca[i, 2];
    iu := 0; iv := 0; Vrh1 := 0; Vrh2 := 0;
    while (iu < StSosed[u]) and (iv < StSosed[v]) do begin
      w := Sosede[PrvaSoseda[u] + iu]; ww := Sosede[PrvaSoseda[v] + iv];
      if w < ww then iu := iu + 1
      else if w > ww then iv := iv + 1
      else begin { Imamo tri daljice, ki tvorijo trikotnik uvw. }
        if Vrh1 = 0 then Vrh1 := w
        else if LeziV(w, u, v, Vrh1) then Vrh1 := w
        else if LeziV(Vrh1, u, v, w) then begin end
        else if Vrh2 = 0 then Vrh2 := w
        else if LeziV(w, u, v, Vrh2) then Vrh2 := w
        else if LeziV(Vrh2, u, v, w) then begin end
        else WriteLn('Napaka: daljice se kriajo!');
        iu := iu + 1; iv := iv + 1;
      end; {if}
    end; {while}
    ShraniTrikotnik(u, v, Vrh1); ShraniTrikotnik(u, v, Vrh2);
  end; {for i}
  
  if ParamCount >= 2 then Assign(T, ParamStr(2)) else Assign(T, 'tri.out');
  Rewrite(T); WriteLn(T, nTri);
  for i := 1 to nTri do begin
    WriteLn(T);
    for j := 0 to 3 do 
      WriteLn(T, Xi[Tri[i, (j mod 3) + 1]]:0:6, ' ', Yi[Tri[i, (j mod 3) + 1]]:0:6);
  end; {for i}
  Close(T);
end. {TrikotnikiIzDaljic}
